/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.konfigass;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.Platform;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CTabFolder;
import org.eclipse.swt.custom.CTabItem;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.ui.IMemento;
import org.eclipse.ui.IPartListener;
import org.eclipse.ui.ISaveablePart2;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IViewReference;
import org.eclipse.ui.IViewSite;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchListener;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;

import de.bsvrz.puk.config.configFile.datamodel.ConfigDataModel;

/**
 * Nach Auswahl des Menpunkts "Betriebsmeldungen..." in der Rahmenapplikation muss sich der Dialog zur Anzeige der
 * Betriebsmeldungen ffnen.
 * 
 * <br>
 * <br>
 * Die Klasse erfllt die technische Anforderungen des Segments Bedienung und Visualisierung (BUV) Modul
 * Konfigurationsassistent: <br>
 * <ul>
 * <li>TAnf-Vi-BW-C3-8</li>
 * </ul>
 * 
 * @version $Revision: 1.27 $ / $Date: 2011/01/17 10:06:19 $ / ($Author: Sans $)
 * 
 */
public class KonfigAssSicht extends ViewPart implements ISaveablePart2
{

  public static final String ID = "de.bsvrz.buv.plugin.konfigass.konfigassView";

  private CTabFolder _tabFolder = null;

  private Composite composite = null;

  private IMemento _memento = null;

  private String _secondViewId;

  protected String _programm;

  protected Combo combo;

  boolean _einmal = false;

  private KonfigAssDarstellungKonfiguration _darstellungKonfigurationComposite;

  private KonfigAssDarstellungAssistenten _darstellungAssisitentenComposite;

//  private ConfigDataModel _config;
  
  private boolean _meldungDatenmodellEntladen = true;

  private IPartListener _iPartListener;

  private IWorkbenchListener _iWorkbenchListener;

  private Composite _parent;
  
  @Override
  public void createPartControl(Composite parent)
  {
    _parent = parent;
    
    restoreState();
    
    //Fr die F1 Taste
    //PlatformUI.getWorkbench().getHelpSystem().setHelp(_parent, Activator.PLUGIN_ID + ".auswahl" );
    PlatformUI.getWorkbench().getHelpSystem().setHelp(_parent, "de.bsvrz.buv.plugin.konfigass" + ".konfigass" );
    
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessVerticalSpace = true;
    gridData.grabExcessHorizontalSpace = true;
    gridData.verticalAlignment = GridData.FILL;
    GridLayout gridLayout = new GridLayout();
    gridLayout.numColumns = 1;
    composite = new Composite(parent, SWT.NONE);
    composite.setLayout(gridLayout);
    composite.setLayoutData(gridData);
    erzeugeCompositeTabFolder();

    erzeugeTabItems();

    _iPartListener = new IPartListener() {
      

      @Override
      public void partActivated(IWorkbenchPart part)
      {
        erzeugeTabItemVonContributor(part.getSite().getPluginId());
        
//        System.out.println("partActivated() " + part.getClass().getName());
        
      }

      @Override
      public void partBroughtToTop(IWorkbenchPart part)
      {
//        System.out.println("partBroughtToTop() " + part.getClass().getName());
      }

      @Override
      public void partClosed(IWorkbenchPart part)
      {         
//        _config = _darstellungKonfigurationComposite.getConfig();
        if (part instanceof KonfigAssSicht)
        {          
          String sec = ((KonfigAssSicht)part).getViewSite().getSecondaryId();
          
          if ((sec != null && sec.equals(_secondViewId)) || (sec == null && _secondViewId == null))
          {
            //String pfadKa = ((KonfigAssSicht)part).getDarstellungKonfigurationComposite()._dateiPfad;
            ConfigDataModel config = ((KonfigAssSicht)part).getDarstellungKonfigurationComposite().getConfig();
            
            if (config != null)
            {  
              if (_meldungDatenmodellEntladen && !getDarstellungKonfigurationComposite().getZugeordnetePluginIds().isEmpty())
              {
                String meldung = Activator.PLUGIN_ID + " : Zugriff der TKA auf das bisherige Datenmodell "
                + "ist nicht mehr mglich, da das Datenmodell entladen wird!";
                
                MessageBox msg = new MessageBox(PlatformUI.getWorkbench().getDisplay().getActiveShell(), SWT.OK
                    | SWT.ICON_WARNING);
                msg.setText("Das Datenmodell wird entladen");
                msg.setMessage(meldung);
                msg.open();
              }  
              
              config.close();
              getDarstellungKonfigurationComposite().setzeKonfigDatenModellFuerTka(null, null);
              if (!((KonfigAssSicht)part).getDarstellungKonfigurationComposite().isDisposed())
              {      
                ((KonfigAssSicht)part).getDarstellungKonfigurationComposite().dispose();
              }
            }            
          }
        }
        else if (part instanceof IViewPart)
        {
          IViewPart vp = (IViewPart)part;
          IViewSite vs = vp.getViewSite();
          
          if (vs.getId().equals("de.bsvrz.buv.plugin.tkabasis.tkaBasisView"))
          {
            String sec = vs.getSecondaryId();
            
            if (sec != null)
            {
              IWorkbench wb = PlatformUI.getWorkbench();
              IWorkbenchWindow ww = wb.getActiveWorkbenchWindow();
              IWorkbenchPage page = ww.getActivePage();
        
              for (IViewReference ref :page.getViewReferences())
              {
                if (ref.getId().equals(KonfigAssSicht.ID))
                {
                  KonfigAssSicht view = (KonfigAssSicht)ref.getView(false);
                  
                  if (view != null)
                  {
                    TableViewer tv = view.getDarstellungAssisitentenComposite().holeTableViewerTeilKonfigurationsAssistenten();              
                    TableItem[] items = tv.getTable().getItems();
                    
                    String ss = sec.toLowerCase();                                
                    String id = "de.bsvrz.buv.plugin.tka" + ss;
                    
                    boolean oeffnenMoeglich = false;
                    
                    for (TableItem ti : items)
                    {                    
                      String tiId = (String)ti.getData("id");
                      
                      if (id.equals(tiId))
                      {
                        ti.setChecked(false);
                        removeZugeordnetePluginId(id);
                        view.getDarstellungKonfigurationComposite().setzeKonfigDatenModellFuerTka(id, null, null);
                      }  
                      
                      if (ti.getChecked())
                        oeffnenMoeglich = true;
                    } 
                                          
                    Button btn = view.getDarstellungAssisitentenComposite().getButtonOeffnen();
                    
                    if (btn != null && !btn.isDisposed()){
                      if (oeffnenMoeglich){
                        btn.setEnabled(true);
                      }
                      else{
                        btn.setEnabled(false);
                      }                        
                    }
                  }
                }  
              }
            }
          }
        }
      }

      @Override
      public void partDeactivated(IWorkbenchPart part)
      {

      }

      @Override
      public void partOpened(IWorkbenchPart part)
      {
//        System.out.println("partOpened() " + part.getClass().getName());
      }

    };
    
    
    
    PlatformUI.getWorkbench().getActiveWorkbenchWindow().getPartService().addPartListener(_iPartListener);
    
    _iWorkbenchListener = new IWorkbenchListener() {
            
      @Override
      public boolean preShutdown(IWorkbench workbench, boolean forced)
      {        
        _meldungDatenmodellEntladen = false;
      
        return true;
      }
      
      @Override
      public void postShutdown(IWorkbench workbench)
      {
        // TODO Auto-generated method stub
        
      }
    };
        
    PlatformUI.getWorkbench().addWorkbenchListener(_iWorkbenchListener);
    
  }
   
  @Override
  public void setFocus()
  {
    //Fr die F1 Taste
    if (_parent != null)
      _parent.setFocus();

  }
  
  
  @Override
  public void dispose()
  {     
    PlatformUI.getWorkbench().getActiveWorkbenchWindow().getPartService().removePartListener(_iPartListener);
    PlatformUI.getWorkbench().removeWorkbenchListener(_iWorkbenchListener);
  }

  /**
   * This method initializes composite
   * 
   */
  private void erzeugeCompositeTabFolder()
  {
    GridData gridData3 = new GridData();
    gridData3.horizontalAlignment = GridData.FILL;
    gridData3.grabExcessHorizontalSpace = true;
    gridData3.grabExcessVerticalSpace = true;
    gridData3.verticalAlignment = GridData.FILL;

    _tabFolder = new CTabFolder(composite, SWT.NONE);
    _tabFolder.setLayout(new GridLayout());
    _tabFolder.setLayoutData(gridData3);
    _tabFolder.addSelectionListener(new SelectionListener() {

      @Override
      public void widgetDefaultSelected(SelectionEvent e)
      {

      }

      @Override
      public void widgetSelected(SelectionEvent e)
      {
        CTabFolder folder = (CTabFolder)e.getSource();
        aktualisiereTabelle(folder);
      }
    });
  }

  public void erzeugeTabItems()
  {
    CTabItem tabItem = new CTabItem(_tabFolder, SWT.NONE);
    tabItem.setText("Konfiguration");

    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.FILL;
    gridData2.grabExcessHorizontalSpace = true;
    gridData2.grabExcessVerticalSpace = true;
    gridData2.verticalAlignment = GridData.FILL;

    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    gridData1.grabExcessHorizontalSpace = true;
    gridData1.grabExcessVerticalSpace = true;
    gridData1.verticalAlignment = GridData.FILL;

    // Composite c1 = new Composite(_tabFolder, SWT.NONE);
    ScrolledComposite c1 = new ScrolledComposite(_tabFolder, SWT.H_SCROLL | SWT.V_SCROLL);
    c1.setLayout(new GridLayout());
    c1.setLayoutData(gridData2);

    _darstellungKonfigurationComposite = new KonfigAssDarstellungKonfiguration(c1, SWT.NONE, _secondViewId);

//    _config = _darstellungKonfigurationComposite.getConfig();

    tabItem.setControl(c1);

    c1.setContent(_darstellungKonfigurationComposite);
    _darstellungKonfigurationComposite.layout();
    c1.setExpandHorizontal(true);
    c1.setExpandVertical(true);
    c1.setMinSize(_darstellungKonfigurationComposite.computeSize(SWT.DEFAULT, SWT.DEFAULT));
    _darstellungKonfigurationComposite.layout();

    CTabItem tabItem2 = new CTabItem(_tabFolder, SWT.NONE);
    tabItem2.setText("Assistenten");

    GridData gridData22 = new GridData();
    gridData22.horizontalAlignment = GridData.FILL;
    gridData22.grabExcessHorizontalSpace = true;
    gridData22.grabExcessVerticalSpace = true;
    gridData22.verticalAlignment = GridData.FILL;

    GridData gridData12 = new GridData();
    gridData12.horizontalAlignment = GridData.FILL;
    gridData12.grabExcessHorizontalSpace = true;
    gridData12.grabExcessVerticalSpace = true;
    gridData12.verticalAlignment = GridData.FILL;

    // Composite c12 = new Composite(_tabFolder, SWT.NONE);
    ScrolledComposite c12 = new ScrolledComposite(_tabFolder, SWT.H_SCROLL | SWT.V_SCROLL);
    c12.setLayout(new GridLayout());
    c12.setLayoutData(gridData22);

    _darstellungAssisitentenComposite = new KonfigAssDarstellungAssistenten(c12, SWT.NONE, _secondViewId);
    tabItem2.setControl(c12);

    c12.setContent(_darstellungAssisitentenComposite);
    _darstellungAssisitentenComposite.layout();
    c12.setExpandHorizontal(true);
    c12.setExpandVertical(true);
    c12.setMinSize(_darstellungAssisitentenComposite.computeSize(SWT.DEFAULT, SWT.DEFAULT));
    _darstellungAssisitentenComposite.layout();

    _tabFolder.setSelection(0);

  }

  @Override
  public void init(IViewSite site, IMemento memento) throws PartInitException
  {
    _memento = memento;
    _secondViewId = site.getSecondaryId();
    super.init(site, memento);
  }

  /**
   * 
   */
  private void restoreState()
  {

    if (_memento == null)
    {
      return;
    }

    _secondViewId = _memento.getString("id");

  }

  @Override
  public void saveState(IMemento memento)
  {
    // TODO Auto-generated method stub
    memento.putString("id", _secondViewId);
    super.saveState(memento);
  }

  /**
   * @param folder
   */
  private void aktualisiereTabelle(CTabFolder folder)
  {
    CTabItem item = folder.getSelection();

    Composite c = (Composite)item.getControl();

    if (c != null)
    {
      if (c.getChildren() != null)
      {
        Control control = c.getChildren()[0];

        if (control instanceof KonfigAssDarstellungKonfiguration)
        {
          _darstellungKonfigurationComposite = (KonfigAssDarstellungKonfiguration)control;
        }
        else if (control instanceof KonfigAssDarstellungAssistenten)
        {
          _darstellungAssisitentenComposite = (KonfigAssDarstellungAssistenten)control;

          List<IKonfigAssDatenModell> listeTKA = _darstellungAssisitentenComposite.holeAngemeldeteAssistenten();

          _darstellungAssisitentenComposite.setInputTabelle(listeTKA);

        }
      }
    }
  }

  public KonfigAssDarstellungKonfiguration getDarstellungKonfigurationComposite()
  {
    return _darstellungKonfigurationComposite;
  }

  public KonfigAssDarstellungAssistenten getDarstellungAssisitentenComposite()
  {
    return _darstellungAssisitentenComposite;
  }

  private void erzeugeTabItemVonContributor(String pluginId)
  {
    IConfigurationElement[] configurationElementsFor = Platform.getExtensionRegistry().getConfigurationElementsFor(
        "de.bsvrz.buv.plugin.konfigass.konfigAssComposite");

    for (IConfigurationElement ce : configurationElementsFor)
    {
      try
      {
        IKonfigAssComposite operation = (IKonfigAssComposite)ce.createExecutableExtension("konfigAssCompositeKlasse");

        if (operation.holePluginId().equals(pluginId))
        {
          if (!_einmal)
          {

            CTabItem tabItem = new CTabItem(_tabFolder, SWT.NONE);
            tabItem.setText(operation.holeName());

            GridData gridData2 = new GridData();
            gridData2.horizontalAlignment = GridData.FILL;
            gridData2.grabExcessHorizontalSpace = true;
            gridData2.grabExcessVerticalSpace = true;
            gridData2.verticalAlignment = GridData.FILL;

            Composite c1 = new Composite(_tabFolder, SWT.NONE);
            c1.setLayout(new GridLayout());
            c1.setLayoutData(gridData2);

            Composite c = operation.holeContributorComposite(c1);
            tabItem.setControl(c);

            _einmal = true;
          }
        }
      }
      catch (CoreException e)
      {
        // TODO Auto-generated catch block
        e.printStackTrace();
      }

    }
  }
  
  public boolean isMeldungDatenmodellEnladen()
  {
    return _meldungDatenmodellEntladen;
  }

  public void setMeldungDatenmodellEnladen(boolean meldungDatenmodellEnladen)
  {
    _meldungDatenmodellEntladen = meldungDatenmodellEnladen;
  }
  
  public void doSave(IProgressMonitor monitor)
  {
  }

  public void doSaveAs()
  {
  }
  
  public boolean isDirty()
  {
    return !getSichtenZuPlugins().isEmpty();
  }
  
  public boolean isSaveAsAllowed()
  {
    return false;
  }

  public boolean isSaveOnCloseNeeded()
  {
    return true;
  }

  public int promptToSaveOnClose()
  {
    Map<String, IViewReference> tkaSichten = getSichtenZuPlugins();
    
    // Zuerst alle dirty Views schliessen
    for (String pluginId: tkaSichten.keySet())
    {
      IViewReference ref = tkaSichten.get(pluginId);
      
      if (ref != null && ref.isDirty())
      {
        IWorkbenchPage page = ref.getPage();
        IWorkbenchPart part = ref.getPart(false);
        page.hideView(ref);
        if (page.isPartVisible(part))
        {
          return ISaveablePart2.CANCEL;
        }  
      }
    }
    
    // tkaSichten aktualisieren - enthlt nur noch nicht-dirty View
    tkaSichten = getSichtenZuPlugins();
  
    // Alle nicht-dirty Views schliessen
    for (String pluginId: tkaSichten.keySet())
    {
      IViewReference ref = tkaSichten.get(pluginId);
      
      if (ref != null)
      {
        ref.getPage().hideView(ref);
      }
    }
    
    return ISaveablePart2.NO;
  }
  
  
//  /**
//   * Alternative mit Zufriff auf statische Methode von EditrManager (Eclipse Internals)
//   */
//  @SuppressWarnings("restriction")
//  public int promptToSaveOnClose()
//  {
//    Map<String, IViewReference> tkaSichten = getSichtenZuPlugins();
//    
//    List<IViewPart> dirtyParts = new ArrayList<IViewPart>();
//    
//    for (String pluginId: tkaSichten.keySet())
//    {
//      IViewReference ref = tkaSichten.get(pluginId);
//      
//      if (ref != null)
//      {
//        IViewPart view = ref.getView(false);
//        if (view instanceof ISaveablePart && ((ISaveablePart)view).isDirty())
//        {  
//          dirtyParts.add(view);
//        }
//      }
//    }
//    
//    if (EditorManager.saveAll(dirtyParts, true, true, false, 
//        PlatformUI.getWorkbench().getActiveWorkbenchWindow()))
//    {
//      for (String pluginId: tkaSichten.keySet())
//      {
//        IViewReference ref = tkaSichten.get(pluginId);
//        
//        if (ref != null)
//        {
//          ref.getPage().hideView(ref);
//        }
//      }
//      return ISaveablePart2.NO;
//    }
//    else
//    {
//      return ISaveablePart2.CANCEL;
//    }
//  }
  
  private Map<String, IViewReference> getSichtenZuPlugins()
  {
    Map<String, IViewReference> result = new HashMap<String, IViewReference>();
    
    IWorkbenchWindow dw = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
    IWorkbenchPage page = dw.getActivePage();
    
    for (String pluginId: getDarstellungKonfigurationComposite().getZugeordnetePluginIds())
    {
      for (IViewReference ref: page.getViewReferences())
      {
        if (ref != null)
        {  
          if (ref.getId().equals("de.bsvrz.buv.plugin.tkabasis.tkaBasisView"))
          {
            if (pluginId.startsWith("de.bsvrz.buv.plugin.tka"))
            {
              String secId = pluginId.substring(pluginId.indexOf("tka") + 3);
              if (ref.getSecondaryId() != null && ref.getSecondaryId().toLowerCase().equals(secId))
              {
                result.put(pluginId, ref);
              }
            } 
          }  
        }
      }  
    }
        
    return result;
  }
  
  void addZugeordnetePluginId(String pluginId)
  {
    boolean dirty = isDirty();
    
    if (getDarstellungKonfigurationComposite().getZugeordnetePluginIds().add(pluginId))
    {
      if (isDirty() != dirty)
      {
        firePropertyChange(PROP_DIRTY);
      }
    }
  }
    
  void removeZugeordnetePluginId(String pluginId)
  {
    boolean dirty = isDirty();
    
    if (getDarstellungKonfigurationComposite().getZugeordnetePluginIds().remove(pluginId))
    {  
      if (isDirty() != dirty)
      {
        firePropertyChange(PROP_DIRTY);
      }
    }
  }
    
  static KonfigAssSicht getView(String secondViewId)
  {
    KonfigAssSicht result = null;
    
    IWorkbenchWindow dw = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
    IWorkbenchPage page = dw.getActivePage();
    
    IViewReference ref = page.findViewReference(KonfigAssSicht.ID, secondViewId);
   
    if (ref != null)
    {
      result = (KonfigAssSicht)ref.getView(false);
    }

    return result;
  }
  
} // @jve:decl-index=0:visual-constraint="10,10,847,295"
