/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.konfigass.assistenten;

import java.io.File;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.IDialogPage;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IWorkbench;

import de.bsvrz.buv.plugin.konfigass.KonfigAssDateiOperationen;

/**
 * Class representing the first page of the wizard
 */

public class KvAoeNeuSeite1 extends WizardPage implements Listener
{
  IWorkbench workbench;

  IStructuredSelection selection;

  Button kvButton;

  Button aoeButton;

  private Label labelBeschreibung;

  private Text textName;

  private Label labelQuelle;

  private Text textPid;

  protected String _name;

  private Label labelKodierung;

  private Text textKodierung;

  private Button backButton = null;

  private Button cancelButton = null;

  /**
   * Constructor for HolidayMainPage.
   */
  public KvAoeNeuSeite1(String pidAoe)
  {
    super("Seit1");
    setTitle("Neuer KV/AOE anlegen");
    setDescription("Schritt 2 : Konfigurationsbereich erstellen");
  }

  /**
   * @see IDialogPage#createControl(Composite)
   */
  public void createControl(Composite parent)
  {
    GridData gd;
    Composite composite = new Composite(parent, SWT.NULL);

    GridLayout gl = new GridLayout();
    int ncol = 3;
    gl.numColumns = ncol;
    composite.setLayout(gl);

    GridData gridData21 = new GridData();
    gridData21.horizontalAlignment = GridData.FILL;
    gridData21.grabExcessHorizontalSpace = true;
    gridData21.verticalAlignment = GridData.CENTER;
    GridData gridData110 = new GridData();
    gridData110.horizontalAlignment = GridData.FILL;
    gridData110.grabExcessHorizontalSpace = true;
    gridData110.verticalAlignment = GridData.CENTER;
    GridData gridData111 = new GridData();
    gridData111.horizontalAlignment = GridData.FILL;
    gridData111.grabExcessHorizontalSpace = true;
    gridData111.verticalAlignment = GridData.CENTER;
    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    gridData1.grabExcessHorizontalSpace = true;
    gridData1.verticalAlignment = GridData.CENTER;

    labelKodierung = new Label(composite, SWT.NONE);
    labelKodierung.setText("Kodierung");
    textKodierung = new Text(composite, SWT.BORDER);
    textKodierung.setLayoutData(gridData111);
    textKodierung.setTextLimit(4);
    textKodierung.addListener(SWT.Verify, new Listener() {
      public void handleEvent(Event e)
      {
        String string = e.text;
        char[] chars = new char[string.length()];
        string.getChars(0, chars.length, chars, 0);
        for (int i = 0; i < chars.length; i++)
        {
          if (!('0' <= chars[i] && chars[i] <= '9'))
          {
            e.doit = false;
            return;
          }
        }
      }
    });

    Label filler1 = new Label(composite, SWT.NONE);

    labelBeschreibung = new Label(composite, SWT.NONE);
    labelBeschreibung.setText("Name");
    textName = new Text(composite, SWT.BORDER);
    textName.setLayoutData(gridData110);
    Label filler = new Label(composite, SWT.NONE);
    labelQuelle = new Label(composite, SWT.NONE);
    labelQuelle.setText("Pid");
    textPid = new Text(composite, SWT.BORDER);
    textPid.setLayoutData(gridData1);
    textPid.setEnabled(false);
    Label filler2 = new Label(composite, SWT.NONE);

    createLine(composite, ncol);

    kvButton = new Button(composite, SWT.RADIO);
    kvButton.setText("Konfigurationsverantwortlicher");
    gd = new GridData(GridData.FILL_HORIZONTAL);
    gd.horizontalSpan = ncol;
    kvButton.setLayoutData(gd);
    kvButton.setEnabled(false);

    aoeButton = new Button(composite, SWT.RADIO);
    aoeButton.setText("Autarke Organisations Einheit");
    gd = new GridData(GridData.FILL_HORIZONTAL);
    gd.horizontalSpan = ncol;
    aoeButton.setLayoutData(gd);
    aoeButton.setEnabled(false);

    // set the composite as the control for this page
    WizardDialog2 container = (WizardDialog2)getContainer();
    backButton = container.getButton(IDialogConstants.BACK_ID);
    cancelButton = container.getButton(IDialogConstants.CANCEL_ID);

    setControl(composite);
    addListeners();
  }

  private void addListeners()
  {
    textKodierung.addListener(SWT.Modify, this);
    textName.addListener(SWT.Modify, this);
    textPid.addListener(SWT.Modify, this);
    backButton.addListener(SWT.Selection, this);
    cancelButton.addListener(SWT.Selection, this);

  }

  @Override
  public IWizardPage getPreviousPage()
  {
    // TODO Auto-generated method stub

    return super.getPreviousPage();
  }

  /**
   * @see Listener#handleEvent(Event)
   */
  public void handleEvent(Event event)
  {
    WizardDialog2 container = (WizardDialog2)getContainer();
    if (container != null){
      Object object = container.getSelectedPage();
      
      if (object instanceof KvAoeNeuStartSeite)
      {
        if (event.widget == backButton)
        {
//          KvAoeNeuAssistent wizard = (KvAoeNeuAssistent)getWizard();
//          KvAoeNeuModell model = wizard.model;
//          if (model._config != null){            
//            model._config.close();
//            model._config = null;
//          }
          
          
          loeschenVerzeichnisMitFortschrittsAnzeige(container);
        }
      }
      else if (object instanceof KvAoeNeuSeite1 || object instanceof KvAoeNeuSeite2 || object instanceof KvAoeNeuSeite3)
      {
        if (event.widget == cancelButton)
        {
          KvAoeNeuAssistent wizard = (KvAoeNeuAssistent)getWizard();
          KvAoeNeuModell model = wizard.model;
          if (model._config != null){            
            model._config.close();
            model._config = null;
          }
          
          loeschenVerzeichnisMitFortschrittsAnzeige(container);
        }        
      }      
    }
//<<<<<<< KvAoeNeuSeite1.java
//    
//    if (getWizard() != null && getWizard().getContainer() != null)
//      getWizard().getContainer().updateButtons(); 
//=======
    if (container != null)
      container.updateButtons();
//>>>>>>> 1.11
  }

  public void loeschenVerzeichnisMitFortschrittsAnzeige(WizardDialog2 container)
  {
    IRunnableWithProgress op2 = new IRunnableWithProgress() {

      @Override
      public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException
      {
        KvAoeNeuAssistent wizard = (KvAoeNeuAssistent)getWizard();
        KvAoeNeuModell model = wizard.model;

        File file = new File(model._zielVerzeichnis);
        File file2 = new File(model._quellVerzeichnis);

        try
        {
          int i = 0;

          int cnt = KonfigAssDateiOperationen.copyDir(file2, file, null, i, null, false);

          KonfigAssDateiOperationen.loescheVerzeichnis(file, monitor, new Integer(cnt));

        }
        catch (IOException e)
        {
          // TODO Auto-generated catch block
          e.printStackTrace();
        }

      }
    };
    try
    {
      container.run(true, true, op2);
    }
    catch (InvocationTargetException e)
    {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
    catch (InterruptedException e)
    {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
  }

  // /*
  // * Returns the next page. Saves the values from this page in the model associated with the wizard. Initializes the
  // * widgets on the next page.
  // */
  //
  public IWizardPage getNextPage()
  {
    saveDataToModel();

    KvAoeNeuAssistent wizard = (KvAoeNeuAssistent)getWizard();
    KvAoeNeuModell model = wizard.model;

    if (model.useAOE)
    {
      KvAoeNeuSeite2 page2 = ((KvAoeNeuAssistent)getWizard()).seite2;
      page2.onEnterPage();
      return page2;
    }
    else
    {
      KvAoeNeuSeite3 page3 = ((KvAoeNeuAssistent)getWizard()).seite3;
      page3.onEnterPage();
      return page3;

    }
  }

  /**
   * @see IWizardPage#canFlipToNextPage()
   */
  public boolean canFlipToNextPage()
  {
    if (getErrorMessage() != null)
      return false;

    if (textKodierung.getText().length() > 0 && textPid.getText().length() > 0 && textName.getText().length() > 0)
    {
      return true;

    }

    return false;
  }

  /*
   * Saves the uses choices from this page to the model. Called on exit of the page
   */
  private void saveDataToModel()
  {

    KvAoeNeuAssistent wizard = (KvAoeNeuAssistent)getWizard();
    KvAoeNeuModell model = wizard.model;

    model._kodierung = textKodierung.getText();
    model._pid = textPid.getText();
    model._name = textName.getText();

  }

  /*
   * Sets the completed field on the wizard class when all the information is entered and the wizard can be completed
   */
  public boolean isPageComplete()
  {
    KvAoeNeuAssistent wizard = (KvAoeNeuAssistent)getWizard();

    if (textKodierung.getText().length() > 0 && textPid.getText().length() > 0 && textName.getText().length() > 0)
    {
      wizard.seite1Complete = true;

      saveDataToModel();
      return true;
    }
    return false;
  }

  /**
   * Applies the status to the status line of a dialog page.
   */
  private void applyToStatusLine(IStatus status)
  {
    String message = status.getMessage();
    if (message.length() == 0)
      message = null;
    switch (status.getSeverity())
    {
      case IStatus.OK:
        setErrorMessage(null);
        setMessage(message);
        break;
      case IStatus.WARNING:
        setErrorMessage(null);
        setMessage(message, WizardPage.WARNING);
        break;
      case IStatus.INFO:
        setErrorMessage(null);
        setMessage(message, WizardPage.INFORMATION);
        break;
      default:
        setErrorMessage(message);
        setMessage(null);
        break;
    }
  }

  // private IStatus findMostSevere()
  // {
  // if (timeStatus.matches(IStatus.ERROR))
  // return timeStatus;
  // if (destinationStatus.matches(IStatus.ERROR))
  // return destinationStatus;
  // if (timeStatus.getSeverity() > destinationStatus.getSeverity())
  // return timeStatus;
  // else
  // return destinationStatus;
  // }

  private static boolean isTextNonEmpty(Text t)
  {
    if (t != null)
    {
      String s = t.getText();
      if ((s != null) && (s.trim().length() > 0))
        return true;
    }
    return false;
  }

  private void createLine(Composite parent, int ncol)
  {
    Label line = new Label(parent, SWT.SEPARATOR | SWT.HORIZONTAL | SWT.BOLD);
    GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
    gridData.horizontalSpan = ncol;
    line.setLayoutData(gridData);
  }

  void onEnterPage()
  {
    // Gets the model
    KvAoeNeuAssistent wizard = (KvAoeNeuAssistent)getWizard();
    KvAoeNeuModell model = wizard.model;

    if (model.useAOE)
    {
      // textKodierung.setEnabled(true);
      aoeButton.setSelection(true);
      kvButton.setSelection(false);
    }
    else
    {
      // textKodierung.setEnabled(false);
      aoeButton.setSelection(false);
      kvButton.setSelection(true);
    }

    if (model._pid != null)
      textPid.setText(model._pid);
  }

}
