/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.konfigass.konsoleAusgabeUpdateListener;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.core.runtime.IStatus;

import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Die Klasse kapselt die Verwaltungsmethoden fr die Listener-Funktionalitt 
 * des {@link KonsoleAusgabeUpdateListener}. Diese Klasse implementiert das Singleton-Pattern
 * und stellt dadurch sicher dass der Listener nur einmal Instanziiert wird. 
 * 
 * 
 * @author Pittner
 *
 */
public class KonsoleAusgabeUpdateListener
{
  /**
   * Liste der {@link IKonsoleAusgabeUpdateListener}, die ber einen neue DaV-Daten informiert werden mchten.
   */
  private LinkedList<IKonsoleAusgabeUpdateListener> _konsoleAusgabeUpdateListener = new LinkedList<IKonsoleAusgabeUpdateListener>();
  
  /**
   * Singleton-Instanz
   */
  private static KonsoleAusgabeUpdateListener _instanz = null;

  /**
   * Liefert die Singletoninstanz.
   * 
   * @return Instanz dieser Klasse.
   */
  public static KonsoleAusgabeUpdateListener getInstanz()
  {
    if (_instanz == null)
    {
      _instanz = new KonsoleAusgabeUpdateListener();
    }
    return _instanz;
  }
    

  /**
   * Fgt einen Listener zur Liste der {@link KonsoleAusgabeUpdateListener} hinzu.
   * 
   * @param listener
   */
  public void addKonsoleAusgabeUpdateListener(IKonsoleAusgabeUpdateListener listener)
  {
    if (!_konsoleAusgabeUpdateListener.contains(listener))
    {
      _konsoleAusgabeUpdateListener.add(listener);
    }
    else
    {
//      System.err
//          .println("Der Listener ist bereits in der Zuhrerliste vorhanden und wird nicht erneut hinzugefgt");
    }
  }
  
  /**
   * Benachrichtigt alle registrierten {@link IKonsoleAusgabeUpdateListener} darber, dass neue Daten vorhanden sind.
   * 
   */
  public void notifyKonsoleAusgabeUpdateListener(IStatus status, long zeitstempel)
  {

    KonsoleAusgabeUpdateEvent event = new KonsoleAusgabeUpdateEvent(this, status, zeitstempel);

    Iterator<IKonsoleAusgabeUpdateListener> it = _konsoleAusgabeUpdateListener.iterator();
    while (it.hasNext())
    {
      IKonsoleAusgabeUpdateListener listener = it.next();
      listener.konsoleAusgabeUpdate(event);
    }
  }
  
  /**
   * Entfernt einen {@link IKonsoleAusgabeUpdateListener} von der Liste der Zuhrer.
   * 
   * @param IKonsoleAusgabeUpdateListener
   *          Zu entfernender {@link IKonsoleAusgabeUpdateListener}.
   */
  public void removeKonsoleAusgabeUpdateListener(IKonsoleAusgabeUpdateListener listener)
  {
    if (_konsoleAusgabeUpdateListener.contains(listener))
    {
      _konsoleAusgabeUpdateListener.remove(listener);
    }
    else
    {
//      System.err.println("Der Listener ist in der Zuhrerliste nicht vorhanden und wird somit nicht entfernt.");
    }
  }


  public void notifyKonsoleAusgabeUpdateListener(String record)
  {
    // TODO Auto-generated method stub
    KonsoleAusgabeUpdateEvent event = new KonsoleAusgabeUpdateEvent(this, record);

    Iterator<IKonsoleAusgabeUpdateListener> it = _konsoleAusgabeUpdateListener.iterator();
    while (it.hasNext())
    {
      IKonsoleAusgabeUpdateListener listener = it.next();
      listener.konsoleAusgabeUpdate(event);
    }
  }
  
}
