/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkaaq;

import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import de.bsvrz.buv.plugin.tkabasis.DataFactory;
import de.bsvrz.buv.plugin.tkabasis.LookupFactory;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.HierarchieObjekt;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchie;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchieObjektTyp;
import de.bsvrz.buv.plugin.tkabasis.regeln.BildungsRegelnBeobachter;
import de.bsvrz.buv.plugin.tkabasis.regeln.BildungsRegelnTools;
import de.bsvrz.buv.plugin.tkabasis.regeln.EingabeVorbelegung;
import de.bsvrz.buv.plugin.tkabasis.regeln.IBildungsRegeln;
import de.bsvrz.dav.daf.communication.dataRepresentation.AttributeBaseValueDataFactory;
import de.bsvrz.dav.daf.communication.dataRepresentation.AttributeHelper;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ObjectLookup;
import de.bsvrz.dav.daf.main.config.SystemObject;

public class BildungsRegelnAllgemeinAQ implements IBildungsRegeln
{
	private IHierarchie _hierarchie;
		
  protected Map<String, String> _praefixe = new HashMap<String, String>();
    
  protected Map<IHierarchieObjektTyp, String[]> _felder = new HashMap<IHierarchieObjektTyp, String[]>();
     
  protected Map<IHierarchieObjektTyp, Map<String, String>> _feldWerte = new HashMap<IHierarchieObjektTyp, Map<String, String>>();
  
  protected Map<String, Map<String, String>> _historie = new HashMap<String, Map<String, String>>();
  
  
  public IHierarchie getHierarchie()
	{
		return _hierarchie;
	}
  
	public void setHierarchie(IHierarchie hierarchie)
	{
		_hierarchie = hierarchie;
	}
  
  protected void initPraefixe()
  {
    _praefixe.put("typ.anzeige", "anzeige");
    _praefixe.put("typ.anzeigeQuerschnitt", "aq");
    _praefixe.put("typ.anzeigeQuerschnittTyp", "aqTyp");
    _praefixe.put("typ.anzeigeTyp", "anzeigeTyp");
//    _praefixe.put("typ.fahrStreifen", "fs");
    _praefixe.put("typ.dirigent", "dirigent");
    _praefixe.put("typ.font", "font");
    _praefixe.put("typ.ligaturen", "ligaturen");
    _praefixe.put("typ.wechseltext", "wechseltext");
    _praefixe.put("typ.wvzInhalt", "wvzInhalt");
    _praefixe.put("typ.wzgInhaltGrafik", "wzgInhaltGrafik");
    _praefixe.put("typ.zeichen", "zeichen");
  }
  
  protected void initFelder()
  {
    _felder.put(HOT_AQ.AnzeigeQuerschnitt, new String[] { "Bezeichnung" });
//    _felder.put(HOT_AQ.AnzeigeQuerschnittSub, new String[] { "Bezeichnung" });
    _felder.put(HOT_AQ.AnzeigeQuerschnittTyp, new String[] { "Bezeichnung" });
    _felder.put(HOT_AQ.Anzeige, new String[] { "Bezeichnung" });
//    _felder.put(HOT_AQ.AnzeigeSub, new String[] { "Bezeichnung" });
    _felder.put(HOT_AQ.AnzeigeTyp, new String[] { "Bezeichnung" });
//    _felder.put(HOT_AQ.FS, new String[] { "Bezeichnung" });
//    _felder.put(HOT_AQ.FSSub, new String[] { "Bezeichnung" });
    _felder.put(HOT_AQ.Dirigent, new String[] { "Bezeichnung" });
    _felder.put(HOT_AQ.Ligaturen, new String[] { "Bezeichnung" });
//    _felder.put(HOT_AQ.LigaturenSub, new String[] { "Bezeichnung" });
    _felder.put(HOT_AQ.WechselText, new String[] { "Bezeichnung" });
    _felder.put(HOT_AQ.WechselText, new String[] { "Bezeichnung" });
    _felder.put(HOT_AQ.WvzInhalt, new String[] { "Bezeichnung" });
    _felder.put(HOT_AQ.WzgInhaltGrafik, new String[] { "Bezeichnung" });
    _felder.put(HOT_AQ.Zeichen, new String[] { "Bezeichnung" });    
//    _felder.put(HOT_AQ.ZeichenSub, new String[] { "Bezeichnung" });    
    _felder.put(HOT_AQ.Zeichensatz, new String[] { "Bezeichnung" });    
//    _felder.put(HOT_AQ.ZeichensatzSub, new String[] { "Bezeichnung" });    
  }
  
  public BildungsRegelnAllgemeinAQ()
  {
    initPraefixe();
    initFelder();
  }
  
  public String getPraefix(String pidTypNeuesObjekt)
  {
    String result = _praefixe.get(pidTypNeuesObjekt);
    
    return result != null ? result : BildungsRegelnTools.LEERER_WERT;
  }
  
  public String[] getEingabeFelder(IHierarchieObjektTyp hotNeuesObjekt)
  {
    String[] result = _felder.get(hotNeuesObjekt);
    
    return result != null ? result : BildungsRegelnTools.LEERE_TEXT_LISTE;
  }
  
  private static EingabeVorbelegung STANDARD_VORBELEGUNG = new EingabeVorbelegung(null, false);
  
  
  public EingabeVorbelegung getEingabeVorbelegung(IHierarchieObjektTyp hotNeuesObjekt, String eingabeFeld, HierarchieObjekt verbundenesHO)
  {    
  	if (Arrays.asList(_felder.get(hotNeuesObjekt)).contains(eingabeFeld))
    {
// Keine Vererbung der Feldinhalte  		  		
//  		if (verbundenesHO != null) 
//	    {
//	      Map<String, String> werte = _historie.get(verbundenesHO.getSystemObjekte().get(0).getPid());
//	      
//	      if (werte != null)
//	      {
//	        String text = werte.get(eingabeFeld);
//	        if (text != null)
//	        {
//	          return new EingabeVorbelegung(text, false);
//	        }
//	      }
//	    }
	    return STANDARD_VORBELEGUNG;
    }  
    
    return BildungsRegelnTools.LEERE_VORBELEGUNG;
  }
  
  public void setWert(IHierarchieObjektTyp hotNeuesObjekt, String eingabeFeld, String wert)
  {
    Map<String, String> werte = _feldWerte.get(hotNeuesObjekt);
    
    if (werte == null)
    {
      werte = new HashMap<String, String>();
      _feldWerte.put(hotNeuesObjekt, werte);
    }
    
    werte.put(eingabeFeld, wert);
  }
  
  public String getWert(IHierarchieObjektTyp hotNeuesObjekt, String eingabeFeld)
  {
    Map<String, String> werte = _feldWerte.get(hotNeuesObjekt);
    
    if (werte == null)
    {
      return BildungsRegelnTools.LEERER_WERT;
    }
    
    String result = werte.get(eingabeFeld);
    
    return result != null ? result : BildungsRegelnTools.LEERER_WERT;
  }
 
  
  public String getPidRegelText(IHierarchieObjektTyp hot)
  {
   	String felder[] = _felder.get(hot);
  	if (felder != null)
  	{
  		String typPid = hot.getTypen().get(0);
  		String result = getPraefix(typPid) + ".Bezeichnung";
  		
  		if (typPid.equals("typ.wvzInhalt") ||
  				typPid.equals("typ.wzgInhaltGrafik") ||
  				typPid.equals("typ.wechseltext") ||
  				typPid.equals("typ.zeichen") ||
  				typPid.equals("typ.ligaturen") ||
  				typPid.equals("typ.anzeige"))
  		{
  			result += ".<PID bergeordnetes Systemobjekt>"; 
  		}
  		else if (typPid.equals("typ.anzeigeQuerschnitt") ||
  				typPid.equals("typ.font"))
  		{
  			result += "[.<PID bergeordnetes Systemobjekt>]"; 
  		}
  		
  		return result;
  	}
  	return BildungsRegelnTools.LEERER_WERT;
  }
  
  public String getNameRegelText(IHierarchieObjektTyp hot)
  {
  	String felder[] = _felder.get(hot);
  	if (felder != null)
  	{
  		return "<PID Systemobjekt>";
  	}
  	return BildungsRegelnTools.LEERER_WERT;
  }
    
  public String[] getPids(IHierarchieObjektTyp hotNeuesObjekt, HierarchieObjekt uebergeordnetesHO)
  {
    
  	String[] result = new String[1];
  	String pidTypNeuesObjekt = hotNeuesObjekt.getTypen().get(0);
  	
    StringBuffer buffer = new StringBuffer();
    
    buffer.append(getPraefix(pidTypNeuesObjekt));
    buffer.append('.');
    
    String bezeichnung = getWert(hotNeuesObjekt, "Bezeichnung");
    buffer.append(BildungsRegelnTools.getPidText(bezeichnung, false, false));
    
    if (uebergeordnetesHO != null)
    {  
      buffer.append('.');
      buffer.append(uebergeordnetesHO.getSystemObjekte().get(0).getPid());
    }
    
    String pid = buffer.toString(); 
    Map<String, String> werte = _feldWerte.get(hotNeuesObjekt);
    if (werte != null && !werte.isEmpty())
    {
      _historie.put(pid, new HashMap<String, String>(werte));
    }
    
    result[0] = pid;
    
    return result; 
  }
  
  public String getName(SystemObject objekt)
  {
  	String pid = objekt.getPid();
  	
  	if (true /*_historie.get(pid) != null*/)
  	{
  		return pid;
  	}
  	
  	return BildungsRegelnTools.LEERER_WERT;
  }
  
  public Data getDatensatz(HierarchieObjekt ho, SystemObject objekt, AttributeGroup atg, Data daten,
  		boolean nurPflichtFelder)
  {
  	if (!ho.getSystemObjekte().contains(objekt))
  	{
  		throw new IllegalArgumentException("Systemobjekt " + objekt.getPid() + 
  				" ist nicht im Hierarchieobjet enthalten");
  	}
  	
  	if (!objekt.getType().getAttributeGroups().contains(atg))
  	{
  		throw new IllegalArgumentException("Attributgruppe " + atg.getName() +
  				" kann nicht mit Systemobjekt " + objekt.getPid() + "verwendet werden "); 
  	}
  	
  	IHierarchie hierarchie = getHierarchie();
  	//ConfigDataModel dataModel = hierarchie.getDataModel();
  	
  	Data data;
  	
  	if (daten == null)
  	{
	    data = DataFactory.createData(atg);
	    // Default und undefiniert Werte setzen
	    data.setToDefault();
  	}
  	else
  	{
  		ObjectLookup lookUp = LookupFactory.createLookupForModifiableVersion(_hierarchie.getDataModel());
      data = DataFactory.createModifiableCopy(daten, lookUp);
  	}
  	
  	if (objekt.getType().getPid().equals("typ.anzeige"))
    {
      if (atg.getPid().equals("atg.anzeige"))
      {
        data.getItem("RealisiertDurch").asReferenceValue().setSystemObject(null);
      }
    }
  	else if (objekt.getType().getPid().equals("typ.anzeigeQuerschnitt"))
  	{
  	  if (atg.getPid().equals("atg.anzeigeQuerschnitt"))
  	  {
  	    // data.getItem("AnzeigeQuerschnittTyp") nicht vorbelegt
  	    data.getItem("ZuHochbauSchrankSchacht").asArray().setLength(0);
  	  }
  	  else if (atg.getPid().equals("atg.anzeigeQuerschnittNba"))
  	  {
  	    data.getItem("VorgelagerterEntscheiderVorhanden").asTextValue().setText("Nein");
  	  }
  	  else if (atg.getPid().equals("atg.punktKoordinaten"))
  	  {
  	    // data.getItem("x") nicht vorbelegt
  	    // data.getItem("y") nicht vorbelegt
  	  }
  	  else if (atg.getPid().equals("atg.punktLiegtAufLinienObjekt"))
  	  {
  	    // data.getItem("LinienReferenz") nicht vorbelegt
  	    data.getItem("Offset").asUnscaledValue().set(0);
  	  }
  	}
  	else if (objekt.getType().getPid().equals("typ.anzeigeQuerschnittTyp"))
    {
      if (atg.getPid().equals("atg.anzeigeQuerschnittTyp"))
      {
        Array array = data.getItem("MglicheBetriebsZustnde").asArray();
        array.setLength(4);
        array.getItem(0).asTextValue().setText("AQNormalbetrieb");
        array.getItem(1).asTextValue().setText("AQBlindbetrieb");
        array.getItem(2).asTextValue().setText("AQVorortbetrieb");
        array.getItem(3).asTextValue().setText("AQTestbetrieb");        
      }
    }
  	else if (objekt.getType().getPid().equals("typ.anzeigeTyp"))
  	{
  	  if (atg.getPid().equals("atg.anzeigeTyp"))
  	  {
  	    data.getItem("KannFreieGrafik").asTextValue().setText("Nein");
  	    data.getItem("AuflsungX").asUnscaledValue().set(0);
  	    data.getItem("AuflsungY").asUnscaledValue().set(0);
  	    data.getItem("Farbdarstellung").asTextValue().setText("Undefiniert");
  	    data.getItem("KannFreiText").asTextValue().setText("Nein");
//  	    data.getItem("TimeoutSchaltBildWechsel").asTimeValue().setSeconds(60L);
  	    data.getItem("TimeoutSchaltBildWechsel").asScaledValue().set(60);
  	    data.getItem("Font").asArray().setLength(0);
  	  }
  	  else if (atg.getPid().equals("atg.anzeigeTypTextAusrichtung"))
      {
//        data.getItem("TextAusrichtung").asTextValue().setText("Linksbndig");
  	    Array array = data.getItem("TextAusrichtung").asArray();
        array.setLength(1);
        array.getItem(0).asTextValue().setText("Linksbndig");
      }
  	}
  	else if (objekt.getType().getPid().equals("typ.font"))
  	{
  	  if (atg.getPid().equals("atg.font"))
  	  {
  	     // data.getItem("Font") nicht vorbelegt
  	     // data.getItem("SchrifthhePixel") nicht vorbelegt
  	     // data.getItem("SchrifthheMillimeter") nicht vorbelegt
  	     // data.getItem("ZeichenAbstandStandard") nicht vorbelegt
  	  }
  	}
  	else if (objekt.getType().getPid().equals("typ.ligaturen"))
  	{
      if (atg.getPid().equals("atg.ligaturen"))
      {
        Array array = data.getItem("Zeichenabstand").asArray();
        array.setLength(1);
        array = data.getItem("Zeichen").asArray();
        array.setLength(2);
      }
  	}
  	else if (objekt.getType().getPid().equals("typ.wechseltext"))
  	{
  	  if (atg.getPid().equals("atg.wechseltext"))
  	  {
  	    // data.getItem("Font") nicht vorbelegt
  	    data.getItem("TextAusrichtung").asTextValue().setText("Linksbndig");
  	    Array array = data.getItem("Textdefinition").asArray();
  	    array.setLength(1);  	     
  	  }
  	}
  	else if (objekt.getType().getPid().equals("typ.wvzInhalt"))
  	{
  	  if (atg.getPid().equals("atg.wvzInhalt"))
  	  {
  	    // data.getItem("Bildinhalt") nicht vorbelegt
  	    data.getItem("Wechseltext").asReferenceValue().setSystemObject(null);
  	    data.getItem("IstGrafik").asTextValue().setText("Nein");
  	    data.getItem("GrafikDarstellungen").asArray().setLength(0);  	      	     
  	  }
  	}
  	else if (objekt.getType().getPid().equals("typ.wzgInhaltGrafik"))
  	{
  	  if (atg.getPid().equals("atg.wzgInhaltGrafik"))
  	  {
  	    data.getItem("PositionX").asUnscaledValue().set(0);
  	    data.getItem("PositionY").asUnscaledValue().set(0);
  	    data.getItem("Skalierung").asScaledValue().set(100.0);
  	    data.getItem("Blinken").asTextValue().setText("Nein");
//      data.getItem("Blinktakt").asTimeValue().setSeconds(0L);
  	    data.getItem("Blinktakt").asUnscaledValue().set(0);  	    
//        data.getItem("URL").asTextValue().setText("Undefiniert");  	      	     
  	  }
  	}
  	else if (objekt.getType().getPid().equals("typ.zeichen"))
    {
      if (atg.getPid().equals("atg.zeichen"))
      {
        // data.getItem("Zeichenname") nicht vorbelegt
        data.getItem("Zeichenbreite").asScaledValue().set(1.0);          
      }
    }
    return data;
  }
  
  public void beobachterAnmelden(BildungsRegelnBeobachter beobachter)
  {
  }
  
  public void alleBeobachterInformieren(String feldName, String wert)
  {
  }
  
  public void beobachterAbmelden(BildungsRegelnBeobachter beobachter)
  {
  }
  
  public void wertGeaendert(String feldName, String wert)
  {
  }

  @Override
  public Data getDatenArrayElement(HierarchieObjekt ho, SystemObject objekt,
  		AttributeGroup atg, Data daten, String arrayName, int arrayIndex,
  		boolean nurPflichtFelder) {
  	// TODO Auto-generated method stub
  	return null;
  }

  @Override
  public boolean setWerte(IHierarchieObjektTyp hotNeuesObjekt, HierarchieObjekt uebergeordnetesHO,
      HierarchieObjekt vorlageHO)
  {
//    String felder[] = _felder.get(hotNeuesObjekt);
//    
//    if (felder != null)
//    { 
//      for (String eingabeFeld: felder)
//      { 
//        // Es werden alle Felder aus uebergeordnetesHO bernommen, vorlageHO wird nicht benutzt
//        EingabeVorbelegung belegung = getEingabeVorbelegung(hotNeuesObjekt, eingabeFeld, uebergeordnetesHO);
//        setWert(hotNeuesObjekt, eingabeFeld, belegung.getWert());
//      }
//    }
//    // TODO Auto-generated method stub
//    return false;
    boolean result = false;
    
    if (!HOT_AQ.FS.equals(hotNeuesObjekt))
    {
      if (uebergeordnetesHO != null)
      {
        EingabeVorbelegung belegung = getEingabeVorbelegung(hotNeuesObjekt, "Bezeichnung", uebergeordnetesHO);
        String wert = belegung.getWert();
      
        Map<IHierarchieObjektTyp, Integer> typMap = new HashMap<IHierarchieObjektTyp, Integer>();
            
        for (HierarchieObjekt ho: getHierarchie().getUntergeordneteObjekte(uebergeordnetesHO))
        {
           IHierarchieObjektTyp hot = ho.getHierarchieObjektTyp();
           Integer anzahl = typMap.get(hot);
           typMap.put(hot, anzahl == null ? 1 : anzahl.intValue() + 1);
        }
        Integer suffix = typMap.get(hotNeuesObjekt);
//        if (suffix != null)
//        {
//          wert += (suffix.intValue() + 1);
//        }
//        
//        setWert(hotNeuesObjekt, "Bezeichnung", wert);
//
//        if (suffix == null)
//        {
//          result = true;
//        }
       
        if (suffix == null)
        {
          setWert(hotNeuesObjekt, "Bezeichnung", wert);
          result = true;
        }
      } 
    }else{
      result = true;
    }
    return result;
    
  }

}
