/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkabasis.hierarchie;

import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import de.bsvrz.dav.daf.main.config.SystemObject;

public class HODatenBeschreibung
{
  private List<SystemObject> _systemObjekte = new Vector<SystemObject>();
  private Map<SystemObject, List<String>> _atg = new HashMap<SystemObject, List<String>>();
  private Map<SystemObject, List<String>> _mengen = new HashMap<SystemObject, List<String>>();
  private Map<SystemObject, List<String>> _defParamAtg = new HashMap<SystemObject, List<String>>();
  
  public List<SystemObject> getSystemObjekte()
  {
    return Collections.unmodifiableList(_systemObjekte);
  }
  
  public void hinzufuegenAtg(SystemObject objekt, String atg)
  {
    List<String> liste = _atg.get(objekt);
    if (liste == null)
    {
      liste = new Vector<String>();
      _atg.put(objekt, liste);
    }
    liste.remove(atg);
    liste.add(atg);
  }
  
  public boolean entfernenAtg(SystemObject objekt, String atg)
  {
    List<String> liste = _atg.get(objekt);
    {
      if (liste != null)
      {
        if (liste.remove(atg))
        {
          if (liste.isEmpty())
          {
            _atg.remove(objekt);
            bereinigen(objekt);
          }
          return true;
        }
      }
    }
    return false;
  }
  
  public List<String> getAtg(SystemObject objekt)
  {
    List<String> result = _atg.get(objekt);
    if (result != null)
    {
      return Collections.unmodifiableList(result);
    }
    else
    {
      return Collections.emptyList();
    }
  }
  
  public void hinzufuegenMenge(SystemObject objekt, String atg)
  {
    List<String> liste = _mengen.get(objekt);
    if (liste == null)
    {
      liste = new Vector<String>();
      _mengen.put(objekt, liste);
    }
    liste.remove(atg);
    liste.add(atg);
  }
  
  public boolean entfernenMenge(SystemObject objekt, String atg)
  {
    List<String> liste = _mengen.get(objekt);
    {
      if (liste != null)
      {
        if (liste.remove(atg))
        {
          if (liste.isEmpty())
          {
            _mengen.remove(objekt);
            bereinigen(objekt);
          }
          return true;
        }
      }
    }
    return false;
  }
  
  public List<String> getMengen(SystemObject objekt)
  {
    List<String> result = _mengen.get(objekt);
    if (result != null)
    {
      return Collections.unmodifiableList(result);
    }
    else
    {
      return Collections.emptyList();
    }
  }
  
  public void hinzufuegenDefParamAtg(SystemObject objekt, String atg)
  {
    List<String> liste = _defParamAtg.get(objekt);
    if (liste == null)
    {
      liste = new Vector<String>();
      _defParamAtg.put(objekt, liste);
    }
    liste.remove(atg);
    liste.add(atg);
  }
  
  public boolean entfernenDefParamAtg(SystemObject objekt, String atg)
  {
    List<String> liste = _defParamAtg.get(objekt);
    {
      if (liste != null)
      {
        if (liste.remove(atg))
        {
          if (liste.isEmpty())
          {
            _defParamAtg.remove(objekt);
            bereinigen(objekt);
          }
          return true;
        }
      }
    }
    return false;
  }
  
  public List<String> getDefParamAtg(SystemObject objekt)
  {
    List<String> result = _defParamAtg.get(objekt);
    if (result != null)
    {
      return Collections.unmodifiableList(result);
    }
    else
    {
      return Collections.emptyList();
    }
  }
  
  private void bereinigen(SystemObject objekt)
  {
    if (!_atg.keySet().contains(objekt) && 
        !_mengen.keySet().contains(objekt) && 
        !_defParamAtg.keySet().contains(objekt))
    {
      _systemObjekte.remove(objekt);
    }
  }
}
