/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkabasis.navigator;

import org.eclipse.jface.viewers.CellLabelProvider;
import org.eclipse.jface.viewers.DecoratingStyledCellLabelProvider;
import org.eclipse.jface.viewers.IDecorationContext;
import org.eclipse.jface.viewers.ILabelDecorator;
import org.eclipse.jface.viewers.StyledString;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.plugin.tkabasis.navigator.navigatorModell.INavigatorElement;

public class NavigatorDecoratingStyledCellLabelProvider extends DecoratingStyledCellLabelProvider
{
  private final IStyledLabelProvider _labelProvider;

  public NavigatorDecoratingStyledCellLabelProvider(IStyledLabelProvider labelProvider, ILabelDecorator decorator,
      IDecorationContext decorationContext)
  {
    super(labelProvider, decorator, decorationContext);
    _labelProvider = labelProvider;
  }

  // falls eigene Tooltips aus dem CellLabelProvider angezeigt werden sollen
  @Override
  public String getToolTipText(Object object)
  {
    if (_labelProvider instanceof CellLabelProvider)
    {
      String toolTip = getStyledText(object).toString();
      String tt = ((CellLabelProvider)_labelProvider).getToolTipText(object);
      if (!tt.isEmpty())
        toolTip += "\n" + tt;
      return toolTip;
    }
    return super.getToolTipText(object);
  }

  @Override
  public Point getToolTipShift(Object object)
  {
    if (_labelProvider instanceof CellLabelProvider)
    {
      return ((CellLabelProvider)_labelProvider).getToolTipShift(object);
    }
    return super.getToolTipShift(object);
  }
  
  @Override
  public int getToolTipTimeDisplayed(Object object)
  {
    if (_labelProvider instanceof CellLabelProvider)
    {
      return ((CellLabelProvider)_labelProvider).getToolTipTimeDisplayed(object);
    }
    return super.getToolTipTimeDisplayed(object);
  }

  @Override
  /**
   * Liefert das dekorierte Bild zum Label
   */
  public Image getImage(Object object)
  {
    Image image = _labelProvider.getImage(object);
    
    ILabelDecorator decorator = PlatformUI.getWorkbench().getDecoratorManager().getLabelDecorator();
    if ( decorator != null && ((INavigatorElement)object).getSuffix() != null )
    {
      Image decorated = decorator.decorateImage(image, object);
      if (decorated != null)
      {
        return decorated;
      }
    }
    return image;
  }
  
  @Override
  /**
   * Liefert den mit einem Suffix dekorierten Text zum Label
   */
  public StyledString getStyledText(Object object)
  {
    StyledString styledString = _labelProvider.getStyledText(object);
    String kbName = ((INavigatorElement)object).getSuffix();
    if ( kbName != null ) 
      styledString.append("  [" + kbName + "]", StyledString.DECORATIONS_STYLER);
    
    return styledString;
  }
  
}
