/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkabasis.navigator;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.ViewerDropAdapter;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DropTargetEvent;
import org.eclipse.swt.dnd.TransferData;

import de.bsvrz.buv.plugin.tkabasis.aktionen.MyAction;
import de.bsvrz.buv.plugin.tkabasis.navigator.NavigatorComposite.DNDModus;
import de.bsvrz.buv.plugin.tkabasis.navigator.navigatorModell.AbstractNavigatorElement;
import de.bsvrz.buv.plugin.tkabasis.navigator.navigatorModell.INavigatorElement;
import de.bsvrz.buv.plugin.tkabasis.navigator.navigatorModell.INavigatorElement.STATUS;

/**
 * Liefert den {@link ViewerDropAdapter} mit dem es mglich wird, in den TreeViewer des
 * {@link NavigatorComposite} {@link Actions}s per Drag and Drop einzufgen. <br>
 * <br>
 * 
 * @author Krause
 * 
 */
public class NavigatorDropListener extends ViewerDropAdapter
{
  /**
   * Flag zum Debuggen
   */
  protected static final boolean _debug = false;
   
  /**
   * Drag-Copy-Aktion, die ausgefhrt werden soll
   */
  private Action _actionDragCopy;

  /**
   * Drag-CopyDeep-Aktion, die ausgefhrt werden soll
   */
  private Action _actionDragCopyDeep;
  
  /**
   * Drag-Move-Aktion, die ausgefhrt werden soll
   */
  private Action _actionDragMove; 
  
  /**
   * Drop-Aktion, die ausgefhrt werden soll
   */
  private MyAction _actionDrop;
  
  /**
   * Shift-Strg-Tasten gedrckt
   */
  private boolean _shiftStrgPressed = false;

  
  public NavigatorDropListener(TreeViewer viewer)
  {
    super(viewer);
  }

//  @Override
  public void dragOver(DropTargetEvent event) 
  {
//    System.out.println("Event-Detail: " + event.detail);
//    System.out.println("Event-Feedback: " + event.feedback);
    
    event.feedback = DND.FEEDBACK_EXPAND | DND.FEEDBACK_SCROLL;

    INavigatorElement target =(INavigatorElement) determineTarget(event);
    if ( target != null && target.isAenderbar() )
    {
      if (target.hasChildren())
        event.feedback |= DND.FEEDBACK_SELECT;
      else
      {
        if (target instanceof AbstractNavigatorElement)
        {
          // falls untergeordnete HOTs mglich sind, Select zulassen
          event.feedback |= DND.FEEDBACK_SELECT;
        }
      }
    }
  }

  
  @Override
  public void drop(DropTargetEvent event)
  {
    int location = determineLocation(event);
    INavigatorElement target =(INavigatorElement) determineTarget(event);
    
    if (_debug)
    {
      String translatedLocation = "";
      switch(location)
      {
        case 1: 
          translatedLocation = "Dropped before the target ";
          break;
        case 2: 
          translatedLocation = "Dropped after the target ";
          break;
        case 3: 
          translatedLocation = "Dropped on the target ";
          break;
        case 4: 
          translatedLocation = "Dropped into nothing ";
          break;
      }
      String dndOperation = "";
      switch(event.detail)
      {
        case 0: 
          dndOperation = "None";
          break;
        case 1: 
          dndOperation = "Copy";
          break;
        case 2: 
          dndOperation = "Move";
          break;
        case 4: 
          dndOperation = "Link";
          break;
        case 16: 
          dndOperation = "Default ";
          break;
      }
//      System.out.println(translatedLocation + target.getName() + " mit Operation: " + dndOperation + "(Wert: " + event.detail + ")");
    }
   
    if (target != null)
    {
      STATUS status = target.getStatus();
      
      // Drop ist nur mglich, wenn Zielobjekt nderbar
      if (status == STATUS.aenderbar /*|| status == STATUS.teilweiseAenderbar*/)
      {
        Action actionDrag = null;
        
        if (_shiftStrgPressed)
        {
          actionDrag = _actionDragCopyDeep;
        }
        else if (event.detail == DND.DROP_MOVE)
        {
          actionDrag = _actionDragMove;
        }
        else if (event.detail == DND.DROP_COPY)
        {
          actionDrag = _actionDragCopy;
        }
        
        if (actionDrag != null && actionDrag.isEnabled())
        {  
          actionDrag.run();
        
          if ( _actionDrop != null/* && target != null */)
          {
            _actionDrop.setData(target);
            _actionDrop.run();
          }
        }
        else
        {
          event.detail = DND.DROP_NONE;
        }
        
        setShiftStrgPressed(false);
      }
      else
      {
        event.detail = DND.DROP_NONE;
      }
    }
    
    super.drop(event);
  }
  
  @Override
  public boolean performDrop(Object data)
  {
    return false;
  }

  @Override
  public boolean validateDrop(Object target, int operation, TransferData transferType)
  {
    //Prfen ob Drop auf diesem Objekt gltig ist
    
    //dies ist ber die Methode dragOver() bzw. _actionDrop.run() in drop() gelst   
    return true;      
  }

  /**
   * Drop-Aktion setzen
   *
   * @param action    Drop-Aktion, die ausgefhrt werden soll
   */
  public void setDropAction(MyAction action)
  {
    if ( action != null )
      _actionDrop = action;
  } 
  
  /**
   * Drag-Aktion fr die entsprechende DND-Modus setzen
   *
   * @param operation DND-Operations-Modus (COPY, COPY_DEEP, MOVE)
   * @param action    Drag-Aktion, die bei der entsprechende Operation ausgefhrt werden soll
   */
  public void setDragAction(DNDModus operation, Action action)
  {
    if ( action != null )
      switch (operation)
      {
        case COPY:
          _actionDragCopy = action;
          break;
        case COPY_DEEP:
          _actionDragCopyDeep = action;
          break;
        case MOVE:
          _actionDragMove = action;
          break;
        default:
          break;
      }
  }
  
  public void setShiftStrgPressed(boolean shiftStrgPressed)
  {
    _shiftStrgPressed = shiftStrgPressed;
//    System.out.println("Shift-Strg: " + _shiftStrgPressed);
  }
  
}
