/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkabasis.navigator.navigatorModell;

import java.util.Collections;
import java.util.List;

import de.bsvrz.buv.plugin.tkabasis.hierarchie.HierarchieObjekt;
import de.bsvrz.dav.daf.main.config.SystemObject;

public class HONavigatorElement extends AbstractNavigatorElement
{
  HierarchieObjekt hierarchieObjekt = null;
  
  public HONavigatorElement(Object parent, String name, KATEGORIE kategorie, HierarchieObjekt ho)
  {
    super(parent, name, kategorie);
    
    this.hierarchieObjekt = ho;
  }
  
  public List<SystemObject> getObjekte()
  {
    if (hierarchieObjekt != null)
    {
      return hierarchieObjekt.getSystemObjekte();
    }
    else
    {
      return Collections.emptyList();
    }
  }
  
  public HierarchieObjekt getHierarchieObjekt()
  {
    return hierarchieObjekt;
  }

  public Object getObjekt()
  {
    return hierarchieObjekt;
  }
  
  @Override
  public String getImagePath()
  {
    // da sich der HierarchieObjektTyp ndern kann, muss hier das Bild immer wieder neu ermittelt und gesetzt werden
    String typ = hierarchieObjekt.getHierarchieObjektTyp().getSymbolName();
    setImagePath(typ);
    setStatus(getStatus());
    return super.getImagePath();
  }
  
  @Override
  public String getName()
  {
    String nameHo = hierarchieObjekt.toString();
    if (!super.getName().equals(nameHo))
      super.setName(nameHo);
    return super.getName();
  }

  @Override
  public String getSuffix()
  {
    // Suffix-Anzeige, wenn Konfigurationsbereich der Hierarchie nicht mit dem des Systemobjektes bereinstimmt
    
    String pidKbSo = hierarchieObjekt.getSystemObjekt(0).getConfigurationArea().getPid();
    HONavigatorElement hoNe = this;
    
    while (hoNe.hasParent() && hoNe.getParent() instanceof HONavigatorElement)
    {
      hoNe = (HONavigatorElement)hoNe.getParent();
    }
    
    String pidKbHo = "";
    if (hoNe.hasParent() && hoNe.getParent() instanceof KBNavigatorElement)    
      pidKbHo = ((KBNavigatorElement)hoNe.getParent()).getKonfigurationsBereich().getPidOrNameOrId();    

    if (pidKbHo != null && !pidKbHo.equals(pidKbSo))
      return pidKbSo;
    
    return null;
  }
  
  @Override
  public String getToolTip()
  {
    // Tooltip-Anzeige zustzlich zur 1.Zeile (Text des Items, der im Decorator hinzugefgt wird): 
    // 2.Zeile enthlt den Name des 1.Systemobjekts   
    
    return hierarchieObjekt.getSystemObjekt(0).getName();

  }
  
}
