/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkabasis.preferences;

import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.help.IWorkbenchHelpSystem;

import de.bsvrz.buv.plugin.tkabasis.Activator;


/**
 * Preferencepage fr die Einstellungen des TKA Verkehr MQ
 * 
 * @author Sans
 * 
 */
public class PreferencePage extends org.eclipse.jface.preference.PreferencePage
		implements IWorkbenchPreferencePage {
	/**
	 * ID der PreferencePage
	 */
	public static final String ID = "de.bsvrz.buv.plugin.tkabasis.preferencePage";

	private Composite _top;
	
	private Group _grpVD;
	private Button _checkVD;
	
	/**
	 * @wbp.parser.constructor
	 */
	public PreferencePage() 
	{
		setPreferenceStore(Activator.getDefault().getPreferenceStore());
	
		String msg = "";
		msg = msg
				+ "Mit Hilfe dieser Seite knnen Einstellungen bzgl. TKA Basis\n";
		msg = msg
				+ "(Teilkonfigurationsassistent Basis-Plugin)  vorgenommen werden\n";

		setDescription(msg);
	}

	public PreferencePage(String title) {
		super(title);
	}

	public PreferencePage(String title, ImageDescriptor image) {
		super(title, image);
	}

	@Override
	protected Control createContents(Composite parent) {
		_top = new Composite(parent, SWT.NONE);

		// Abfrage Verwerfen von Daten
		_grpVD = new Group(_top, SWT.NONE);
		_grpVD.setText("Sicherheitsabfrage beim Verwerfen von Daten");

		_checkVD = new Button(_grpVD, SWT.CHECK);	
		_checkVD.setText("Sicherheitsabfrage anzeigen");
		

		/*
		 * Layout
		 */
		GridData gdTop = new GridData();
		gdTop.grabExcessHorizontalSpace = true;
		gdTop.horizontalAlignment = GridData.FILL;
		_top.setLayoutData(gdTop);

		GridLayout glTop = new GridLayout(1, false);
		_top.setLayout(glTop);

    // Abfrage Verwerfen von Daten
		_grpVD.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		GridLayout glGrpVD = new GridLayout(4, false);
		_grpVD.setLayout(glGrpVD);
		
		
		/*
		 * Defaultwerte setzen
		 */
		actionLoadFromPreferenceStore();

		return null;
	}

	/**
	 * Stellt die Default-Einstellungen aus den Prferenzen wieder her.
	 */
	private void actionLoadDefaultsFromPreferenceStore() {
		
		IPreferenceStore preferences = Activator.getDefault()
				.getPreferenceStore();
				
		//Prfung beim Lschen
		boolean selectedVD = true;
		if (preferences.contains(DefsPara.AbfrageDatenVerwerfen))	
				selectedVD = preferences.getDefaultBoolean(DefsPara.AbfrageDatenVerwerfen);
		
		if (_checkVD != null) {
			_checkVD.setSelection(selectedVD);
		}
	}


	/**
	 * Stellt die Einstellungen aus den Prferenzen wieder her. Wenn keine
	 * gesetzt wurden, werden die Defaulteinstellungen wiederhergestellt.
	 */
	private void actionLoadFromPreferenceStore() {

		IPreferenceStore preferences = Activator.getDefault()
				.getPreferenceStore();
		
		boolean selectedVD =  preferences.getBoolean(DefsPara.AbfrageDatenVerwerfen);
		
		if (_checkVD != null) {
		  _checkVD.setSelection(selectedVD);
		}
	
	}

	/**
	 * Speichert die i.d. PreferencePage gemachten Einstellungen in den
	 * Prferenzen
	 */
	private void actionSaveToPreferenceStore()
	{
		boolean allValuesAvailable = true;
		boolean selectedVD = false;
		
		if (_checkVD != null)
		{
		  selectedVD = _checkVD.getSelection();
		}
	  else 
	  {
			allValuesAvailable = false;
		}
		
		if (allValuesAvailable) 
		{
			IPreferenceStore preferences = Activator.getDefault()
					.getPreferenceStore();
		
			preferences.setValue(DefsPara.AbfrageDatenVerwerfen, selectedVD);
		}

	}
	

	@Override
	protected void performApply() {
		actionSaveToPreferenceStore();
	}

	@Override
	public boolean performCancel() {
		return super.performCancel();
	}

	@Override
	protected void performDefaults() {
		actionLoadDefaultsFromPreferenceStore();
		super.performDefaults();
	}

	@Override
	public boolean performOk() {
		performApply();
		return super.performOk();
	}

	@Override
	public void init(IWorkbench workbench) {

	}

	@Override
	public void performHelp() {
		String helpContextId = Activator.PLUGIN_ID + ".preferencepage";
		IWorkbenchHelpSystem helpSystem = PlatformUI.getWorkbench()
				.getHelpSystem();
		// helpSystem.displayHelp(helpContextId);
		// helpSystem.displayHelp(helpContextId);
	}

}
