/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkamqgrp;

import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import de.bsvrz.buv.plugin.tkabasis.hierarchie.ArrayInfo;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.AtgInfo;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.AttributInfo;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IAttributInfo;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchieObjektTyp;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.MengeInfo;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.TypInfo;
import de.bsvrz.dav.daf.main.config.SystemObjectType;

/**
 * Hierachieobjekt-Typen Verkehr MQ
 *   
 * @author Sans
 *
 */
public enum HOT_MQGrp implements IHierarchieObjektTyp
{
  MessStellenGruppe("MessStellenGruppe", new TypInfo[] { new TypInfo("typ.messStellenGruppe",	new AtgInfo[] 
	  {
	    new AtgInfo("atg.messStellenGruppe", true, new IAttributInfo[]
	    {
	      new ArrayInfo("MessStellen", true, true, true, false, false),      
	      new AttributInfo("SystematischeDetektorfehler", true, false, true, true, true, true)
	    }),
	  }, 
    null) // Keine Mengen definiert
  }),
  MessStelle("MessStelle", new TypInfo[] { new TypInfo("typ.messStelle", 
  	new AtgInfo[] 
	  {
	    new AtgInfo("atg.messStelle", true, new IAttributInfo[]
	    {
	      new AttributInfo("Prfling", true, false, true, true, true, false)
	    }),
	  }, 
   new MengeInfo[] { 
     new MengeInfo("menge.abfahrten", "Abfahrten", false, null),  
     new MengeInfo("menge.zufahrten", "Zufahrten", false, null)       
     })  
  }),
  MQPruefling("MQ_Pruefling", new TypInfo[] { new TypInfo("typ.messQuerschnitt", 
    null,  // Keine Attributgruppen definiert
    null ) // Keine Mengen definiert        
	}, 
  true),	    	        
  MQZufahrt("MQ_Zufahrt", new TypInfo[] { new TypInfo("typ.messQuerschnitt", 
		  null,  // Keine Attributgruppen definiert
		  null ) // Keine Mengen definiert        
  }, 
  true),	    	        
  MQAbfahrt("MQ_Abfahrt", new TypInfo[] { new TypInfo("typ.messQuerschnitt", 
		  null,  // Keine Attributgruppen definiert
		  null ) // Keine Mengen definiert        
  }, 
  true);	    	        
  
  /**
   * Name des HOT - wird in der Navigationsdarstellung verwendet
   */
  private String _name;
  
  /**
   * Die Systemobjekt-Typen, die von diesem HOT verwendet werden
   */
  private List<String> _typen = new Vector<String>();
  
  /**
   * Die Typinfos zu den Systemobjekttypen
   */
  private Map<String, TypInfo> _typInfos = new HashMap<String, TypInfo>();

private boolean _bildungsRegelGefordert;
  
    
  /**
   * Erzeugen eines neuen HOT_MQ
   * 
   * @param name Name des HOT
   * @param typInfos Systemobjekt-Typen und zugehrige Informationen, 
   * die von diesem HOT verwendet werden
   */
  private HOT_MQGrp(String name, TypInfo[] typInfos)
  {
    _name = name;
    if (typInfos != null)
    {  
    	for (TypInfo  info: typInfos)
    	{	
    		_typen.add(info.getTyp());
    		_typInfos.put(info.getTyp(), info);
    	}	
    }  
  }
  
  /**
   * Erzeugen eines neuen HOT_MQGrp
   * 
   * @param name Name des HOT
   * @param typInfos Systemobjekt-Typen und zugehrige Informationen, 
   * die von diesem HOT verwendet werden
   */
  private HOT_MQGrp(String name, TypInfo[] typInfos, boolean bildungsRegelGefordert)
  {
    this(name, typInfos); 
    _bildungsRegelGefordert = bildungsRegelGefordert;
  }
  
  /**
   * Text-Reprsentation fr diesen HOT
   * 
   * @return den Namen des HOT - wird in der Navigationsdarstellung verwendet
   */
  @Override
  public String toString()
  {
    return getName();
  }
    
  /**
	 * @see IHierarchieObjektTyp#getName()
	 */
  public String getName()
  {
    return _name;
  }
  
  /**
	 * @see IHierarchieObjektTyp#getSymbolName()
	 */
  public String getSymbolName()
  {
  	return super.toString();
  }
  
  /**
	 * @see IHierarchieObjektTyp#getTypen()
	 */
  public List<String> getTypen()
  {
    return Collections.unmodifiableList(_typen); 
  }
  
  /**
	 * @see IHierarchieObjektTyp#getTypInfo(String)
	 */
  public TypInfo getTypInfo(String typ)
  {
  	return _typInfos.get(typ);
  }
  
  /**
	 * @see IHierarchieObjektTyp#getTypInfo(SystemObjectType)
	 */
  public TypInfo getTypInfo(SystemObjectType sot)
  {
  	TypInfo result = getTypInfo(sot.getPid());
  	
  	if (result == null && !sot.isBaseType())
  	{
  		for (SystemObjectType superTyp: sot.getSuperTypes())
  		{
  			if (superTyp.isConfigurating())
  			{
  				 result = getTypInfo(superTyp);
  				 if (result != null)
  				 {
  					 break;
  				 }
  			}
  		}
  	}
  	
  	return result;
  }

  @Override
  public boolean isSystemObjektVorhanden()
  {
    // TODO Auto-generated method stub    
    return _bildungsRegelGefordert;
  }
  
  public boolean isSystemObjektNichtAenderbar()
  {
  	return isSystemObjektVorhanden();
  }
  
}	
