/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkanetz;

import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchieObjektTyp;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.MengeInfo;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.TypInfo;
import de.bsvrz.dav.daf.main.config.SystemObjectType;

/**
 * Hierachieobjekt-Typen Netzbestandteile
 *   
 * @author Krause
 *
 */
public enum HOT_Netz implements IHierarchieObjektTyp
{

  VerkehrsModellNetz("VerkehrsModellNetz", new TypInfo[] { new TypInfo("typ.verkehrsModellNetz", 
      null,  // Keine Attributgruppen definiert 
      new MengeInfo[] 
      { 
        new MengeInfo("menge.netzBestandTeile", "NetzBestandTeile", false, null),
        new MengeInfo("menge.aktionen", "Aktionen", true, null),
        new MengeInfo("menge.baustellen", "Baustellen", true, null),
        new MengeInfo("menge.situationen", "Situationen", true, null),
        new MengeInfo("menge.staus", "Staus", true, null),
        new MengeInfo("menge.unflle", "Unflle", true, null),
      })
  }),
  SchwachStelle("SchwachStelle", new TypInfo[] { new TypInfo("typ.schwachStelle",	
      null,  // Keine Attributgruppen definiert
      new MengeInfo[] { new MengeInfo("menge.netzBestandTeile", "NetzBestandTeile", false, null) })
  }),
  VerkehrsModellNetzReferenz("VerkehrsModellNetz (Referenz)", new TypInfo[] { new TypInfo("typ.verkehrsModellNetz", 
      null,  // Keine Attributgruppen definiert 
      new MengeInfo[] 
      { 
        new MengeInfo("menge.netzBestandTeile", "NetzBestandTeile", false, null),
        new MengeInfo("menge.aktionen", "Aktionen", true, null),
        new MengeInfo("menge.baustellen", "Baustellen", true, null),
        new MengeInfo("menge.situationen", "Situationen", true, null),
        new MengeInfo("menge.staus", "Staus", true, null),
        new MengeInfo("menge.unflle", "Unflle", true, null),
      })
  },
  true, false),
  SchwachStelleReferenz("SchwachStelle (Referenz)", new TypInfo[] { new TypInfo("typ.schwachStelle",  
      null,  // Keine Attributgruppen definiert
      new MengeInfo[] { new MengeInfo("menge.netzBestandTeile", "NetzBestandTeile", false, null) })
  },
  true, false),
  ASS("ueresStraenSegment", new TypInfo[] 
  { 
//     new TypInfo("typ.ueresStraenSegment", new AtgInfo[] 
//     {
//         //TODO Workaround um AeueresStraenSegmente anlegen zu knnen,spter wieder raus
//         new AtgInfo("atg.bestehtAusLinienObjekten", false, new IAttributInfo[]
//         {
  		
//             new ArrayInfo("LinienReferenz", true, false, false, true, true)  
//         }),
//     },
      
       new TypInfo("typ.ueresStraenSegment", 
         null,  // Keine Attributgruppen definiert
         null ) // Keine Mengen definiert
  },
  true, true); 

  
  /**
   * Name des HOT - wird in der Navigationsdarstellung verwendet
   */
  private String _name;
  
  /**
   * Die Systemobjekt-Typen, die von diesem HOT verwendet werden
   */
  private List<String> _typen = new Vector<String>();
  
  /**
   * Die Typinfos zu den Systemobjekttypen
   */
  private Map<String, TypInfo> _typInfos = new HashMap<String, TypInfo>();
  
  /**
   * Flagge die anzeigt ob das SystemObjekt bereits vorhanden ist und nur ausgewhlt werden soll
   */
  private boolean _isSystemObjektVorhanden = false;
  
  /**
   * Flagge die anzeigt ob das SystemObjekt nicht nderbar ist
   */
  private boolean _isSystemObjektNichtAenderbar = false;
    
  /**
   * Erzeugen eines neuen HOT_Netz
   * 
   * @param name Name des HOT
   * @param typInfos Systemobjekt-Typen und zugehrige Informationen, 
   * die von diesem HOT verwendet werden
   */
  private HOT_Netz(String name, TypInfo[] typInfos)
  {
    _name = name;
    if (typInfos != null)
    {  
    	for (TypInfo  info: typInfos)
    	{	
    		_typen.add(info.getTyp());
    		_typInfos.put(info.getTyp(), info);
    	}	
    }  
  }
  
  /**
   * Erzeugen eines neuen HOT_Netz
   * 
   * @param name
   *          Name des HOT
   * @param typInfos
   *          Systemobjekt-Typen und zugehrige Informationen, die von diesem
   *          HOT verwendet werden
   * @param isSystemObjektVorhanden
   *          Flag, ob das SystemObjekt bereits vorhanden ist
   */
  private HOT_Netz( String name, TypInfo[] typInfos, boolean isSystemObjektVorhanden ,
  		boolean isSystemObjektNichtAenderbar)
  {
    this( name, typInfos );
    _isSystemObjektVorhanden = isSystemObjektVorhanden;
    _isSystemObjektNichtAenderbar = isSystemObjektNichtAenderbar;
  }
  
  /**
   * Text-Reprsentation fr diesen HOT
   * 
   * @return den Namen des HOT - wird in der Navigationsdarstellung verwendet
   */
  @Override
  public String toString()
  {
    return getName();
  }
    
  /**
	 * @see IHierarchieObjektTyp#getName()
	 */
  public String getName()
  {
    return _name;
  }
  
  /**
	 * @see IHierarchieObjektTyp#getSymbolName()
	 */
  public String getSymbolName()
  {
  	return super.toString();
  }
  
  /**
	 * @see IHierarchieObjektTyp#getTypen()
	 */
  public List<String> getTypen()
  {
    return Collections.unmodifiableList(_typen); 
  }
  
  /**
	 * @see IHierarchieObjektTyp#getTypInfo(String)
	 */
  public TypInfo getTypInfo(String typ)
  {
  	return _typInfos.get(typ);
  }
  
  /**
	 * @see IHierarchieObjektTyp#getTypInfo(SystemObjectType)
	 */
  public TypInfo getTypInfo(SystemObjectType sot)
  {
  	TypInfo result = getTypInfo(sot.getPid());
  	
  	if (result == null && !sot.isBaseType())
  	{
  		for (SystemObjectType superTyp: sot.getSuperTypes())
  		{
  			if (superTyp.isConfigurating())
  			{
  				 result = getTypInfo(superTyp);
  				 if (result != null)
  				 {
  					 break;
  				 }
  			}
  		}
  	}
  	
  	return result;
  }

  @Override
  public boolean isSystemObjektVorhanden()
  {
    return _isSystemObjektVorhanden;
  }
  
  public boolean isSystemObjektNichtAenderbar()
  {
    return _isSystemObjektNichtAenderbar;
  }
  
}	
