/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkatls;

import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import de.bsvrz.buv.plugin.tkabasis.hierarchie.ArrayInfo;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.AtgInfo;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.AttributInfo;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IAttributInfo;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchieObjektTyp;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.MengeInfo;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.TypInfo;
import de.bsvrz.dav.daf.main.config.SystemObjectType;

/**
 * Hierachieobjekt-Typen TLS
 *   
 * @author Sans
 *
 */
public enum HOT_TLS implements IHierarchieObjektTyp
{
  VIZ("VIZ", new TypInfo[] 
 	{ 
   	new TypInfo("typ.viz", new AtgInfo[] 
   	{
  	    new AtgInfo("atg.gert", true, new IAttributInfo[]
  	    {
  	      new AttributInfo("Bezeichnung",true, false, true, true, true, true),
  	      new AttributInfo("KnotenNummer", true, true, false, true, true, false),
  	      new AttributInfo("UmsetzungsModul", false, true, true, true, true, false)
  	    }),
  	    new AtgInfo("atg.gertZustndigkeit", false, new IAttributInfo[]
   	    {
  	    		new ArrayInfo("Gert", false, false, false, true, true),
   	    }),
  	    new AtgInfo("atg.punktKoordinaten", false, new IAttributInfo[]
  	    {
  	      new AttributInfo("x", true, false, false, true, true, false),
  	      new AttributInfo("y", true, false, false, true, true, false)
  	    }),
  	  }, 
      new MengeInfo[] { new MengeInfo("menge.anschlussPunkteGert", "AnschlussPunkteGert", false, null) 
    }),
   }),
   
  VRZ("VRZ",new TypInfo[] 
 	{ 
   	new TypInfo("typ.vrz", new AtgInfo[] 
   	{
  	    new AtgInfo("atg.gert", true, new IAttributInfo[]
  	    {
  	      new AttributInfo("Bezeichnung",true, false, true, true, true, true),
  	      new AttributInfo("KnotenNummer", true, true, false, true, true, false),
  	      new AttributInfo("UmsetzungsModul", false, true, true, true, true, false)
  	    }),
  	    new AtgInfo("atg.gertZustndigkeit", false, new IAttributInfo[]
   	    {
  	    		new ArrayInfo("Gert", false, false, false, true, true),
   	    }),
  	    new AtgInfo("atg.punktKoordinaten", false, new IAttributInfo[]
  	    {
  	      new AttributInfo("x", true, false, false, true, true, false),
  	      new AttributInfo("y", true, false, false, true, true, false)
  	    }),
  	  }, 
      new MengeInfo[] { new MengeInfo("menge.anschlussPunkteGert", "AnschlussPunkteGert", false, null) 
    }),
   
   	new TypInfo("typ.anschlussPunkt", new AtgInfo[] 
   	{
  	    new AtgInfo("atg.anschlussPunkt", true, new IAttributInfo[]
  	    {
  	      new AttributInfo("Bezeichnung",true, false, true, true, true, true),
  	      new AttributInfo("ProtokollTyp", false, false, true, true, true, true),
  	      new AttributInfo("PortNummer", true, true, true, true, true, true)
  	    }),
  	  }, 
      new MengeInfo[] { new MengeInfo("menge.anschlussPunkteKommunikationsPartner", 
      		"AnschlussPunkteKommunikationsPartner", false, null) 
    }),
     new TypInfo("typ.anschlussPunktKommunikationsPartner", new AtgInfo[] 
 		{
 		    new AtgInfo("atg.anschlussPunktKommunikationsPartner", true, new IAttributInfo[]
 		    {
 		      new AttributInfo("Bezeichnung",true, false, true, true, true, true),
 		      new AttributInfo("ProtokollTyp", false, false, true, true, true, true),
 		      new AttributInfo("KommunikationsPartner", true, false, true, false, false, true),
 		      new AttributInfo("PortNummer", true, true, true, true, true, true)
 		    }),
 		  }, 
 		  null), // Keine Mengen definiert
 	  
   }),
   
  UZ("UZ",new TypInfo[] 
	{ 
  	new TypInfo("typ.uz", new AtgInfo[] 
  	{
 	    new AtgInfo("atg.gert", true, new IAttributInfo[]
 	    {
 	      new AttributInfo("Bezeichnung",true, false, true, true, true, true),
 	      new AttributInfo("KnotenNummer", true, true, false, true, true, false),
 	      new AttributInfo("UmsetzungsModul", false, true, true, true, true, false)
 	    }),
 	    new AtgInfo("atg.gertZustndigkeit", false, new IAttributInfo[]
  	    {
 	    		new ArrayInfo("Gert", false, false, false, true, true),
  	    }),
 	    new AtgInfo("atg.punktKoordinaten", false, new IAttributInfo[]
 	    {
 	      new AttributInfo("x", true, false, false, true, true, false),
 	      new AttributInfo("y", true, false, false, true, true, false)
 	    }),
 	  }, 
     new MengeInfo[] { new MengeInfo("menge.anschlussPunkteGert", "AnschlussPunkteGert", false, null) 
   }),
  
  	new TypInfo("typ.anschlussPunkt", new AtgInfo[] 
  	{
 	    new AtgInfo("atg.anschlussPunkt", true, new IAttributInfo[]
 	    {
 	      new AttributInfo("Bezeichnung",true, false, true, true, true, true),
 	      new AttributInfo("ProtokollTyp", false, false, true, true, true, true),
 	      new AttributInfo("PortNummer", true, true, true, true, true, true)
 	    }),
 	  }, 
     new MengeInfo[] { new MengeInfo("menge.anschlussPunkteKommunikationsPartner", 
     		"AnschlussPunkteKommunikationsPartner", false, null) 
   }),
    new TypInfo("typ.anschlussPunktKommunikationsPartner", new AtgInfo[] 
		{
		    new AtgInfo("atg.anschlussPunktKommunikationsPartner", true, new IAttributInfo[]
		    {
		      new AttributInfo("Bezeichnung",true, false, true, true, true, true),
		      new AttributInfo("ProtokollTyp", false, false, true, true, true, true),
		      new AttributInfo("KommunikationsPartner", true, false, true, false, false, true),
		      new AttributInfo("PortNummer", true, true, true, true, true, true)
		    }),
		  }, 
		  null), // Keine Mengen definiert
	  
  }),
  
  KRI("KRI",new TypInfo[] 
	{ 
  	new TypInfo("typ.kri", new AtgInfo[] 
  	{
 	    new AtgInfo("atg.gert", true, new IAttributInfo[]
 	    {
 	      new AttributInfo("Bezeichnung",true, false, true, true, true, true),
 	      new AttributInfo("KnotenNummer", true, true, false, true, true, false),
 	      new AttributInfo("UmsetzungsModul", false, true, true, true, true, false)
 	    }),
 	    new AtgInfo("atg.gertZustndigkeit", false, new IAttributInfo[]
  	    {
 	    		new ArrayInfo("Gert", false, false, false, true, true),
  	    }),
 	    new AtgInfo("atg.punktKoordinaten", false, new IAttributInfo[]
 	    {
 	      new AttributInfo("x", true, false, false, true, true, false),
 	      new AttributInfo("y", true, false, false, true, true, false)
 	    }),
 	  }, 
     new MengeInfo[] { new MengeInfo("menge.anschlussPunkteGert", "AnschlussPunkteGert", false, null) 
   }),
  
  	new TypInfo("typ.anschlussPunkt", new AtgInfo[] 
  	{
 	    new AtgInfo("atg.anschlussPunkt", true, new IAttributInfo[]
 	    {
 	      new AttributInfo("Bezeichnung",true, false, true, true, true, true),
 	      new AttributInfo("ProtokollTyp", false, false, true, true, true, true),
 	      new AttributInfo("PortNummer", true, true, true, true, true, true)
 	    }),
 	  }, 
     new MengeInfo[] { new MengeInfo("menge.anschlussPunkteKommunikationsPartner", 
     		"AnschlussPunkteKommunikationsPartner", false, null) 
   }),
    new TypInfo("typ.anschlussPunktKommunikationsPartner", new AtgInfo[] 
		{
		    new AtgInfo("atg.anschlussPunktKommunikationsPartner", true, new IAttributInfo[]
		    {
		      new AttributInfo("Bezeichnung",true, false, true, true, true, true),
		      new AttributInfo("ProtokollTyp", false, false, true, true, true, true),
		      new AttributInfo("KommunikationsPartner", true, false, true, false, false, true),
		      new AttributInfo("PortNummer", true, true, true, true, true, true)
		    }),
		  }, 
		  null), // Keine Mengen definiert
	  
  }),
  
  Inselbus("Inselbus",new TypInfo[] 
 	{ 
   	new TypInfo("typ.anschlussPunkt", new AtgInfo[] 
   	{
  	    new AtgInfo("atg.anschlussPunkt", true, new IAttributInfo[]
  	    {
  	      new AttributInfo("Bezeichnung",true, false, true, true, true, true),
  	      new AttributInfo("ProtokollTyp", false, false, true, true, true, true),
  	      new AttributInfo("PortNummer", true, true, true, true, true, true)
  	    }),
  	  }, 
      new MengeInfo[] { new MengeInfo("menge.anschlussPunkteKommunikationsPartner", 
      		"AnschlussPunkteKommunikationsPartner", false, null) 
    }),
  }),
   
  SM("SM",new TypInfo[] 
  { 
		new TypInfo("typ.steuerModul", new AtgInfo[] 
   	{
  	    new AtgInfo("atg.gert", true, new IAttributInfo[]
  	    {
  	      new AttributInfo("Bezeichnung",true, false, true, true, true, true),
  	      new AttributInfo("KnotenNummer", true, true, false, true, true, false),
  	      new AttributInfo("UmsetzungsModul", false, false, true, true, true, false)
  	    }),
  	    new AtgInfo("atg.gertZustndigkeit", false, new IAttributInfo[]
   	    {
  	    		new ArrayInfo("Gert", false, false, false, true, true),
   	    }),
  	    new AtgInfo("atg.punktKoordinaten", false, new IAttributInfo[]
  	    {
  	      new AttributInfo("x", true, false, false, true, true, false),
  	      new AttributInfo("y", true, false, false, true, true, false)
  	    }),
  	  }, 
      new MengeInfo[] { new MengeInfo("menge.eak", "Eak", false, null),
				new MengeInfo("menge.anschlussPunkteGert", "AnschlussPunkteGert", false, null)
    }),
    new TypInfo("typ.anschlussPunktKommunikationsPartner", new AtgInfo[] 
		{
		    new AtgInfo("atg.anschlussPunktKommunikationsPartner", true, new IAttributInfo[]
		    {
		      new AttributInfo("Bezeichnung",true, false, true, true, true, true),
		      new AttributInfo("ProtokollTyp", false, false, true, true, true, true),
		      new AttributInfo("KommunikationsPartner", true, false, true, false, false, true),
		      new AttributInfo("PortNummer", true, true, true, true, true, true)
		    }),
		  }, 
		  null), // Keine Mengen definiert
  }),
   
  EAK("EAK",new TypInfo[] 
     { new TypInfo("typ.eak", new AtgInfo[] 
                                                 	{
   	    new AtgInfo("atg.eak", true, new IAttributInfo[]
   	    {
   	      new AttributInfo("Bezeichnung",true, false, true, true, true, true),
   	      new AttributInfo("OSI2Adresse", true, true, false, true, true, true),
   	      new AttributInfo("UmsetzungsModul", false, false, true, true, true, false)
   	    }),
   	    new AtgInfo("atg.punktKoordinaten", false, new IAttributInfo[]
   	    {
   	      new AttributInfo("x", true, false, false, true, true, false),
   	      new AttributInfo("y", true, false, false, true, true, false)
   	    }),
   	  }, 
       new MengeInfo[] { new MengeInfo("menge.de", "De", false, null) 
     }),
  }),
  
  DE("DE",new TypInfo[] 
     { new TypInfo("typ.de", new AtgInfo[] 
                                        	{
   	    new AtgInfo("atg.de", true, new IAttributInfo[]
   	    {
   	      new AttributInfo("Bezeichnung",true, false, true, true, true, true),
   	      new AttributInfo("Cluster", true, false, true, true, true, true),
   	      new AttributInfo("EAKanal", true, true, true, true, true, true),
   	      new AttributInfo("DEKanal", true, true, true, true, true, true),
   	      new AttributInfo("UmsetzungsModul", false, false, true, true, true, true)
   	    }),
   	    new AtgInfo("atg.punktKoordinaten", false, new IAttributInfo[]
   	    {
   	      new AttributInfo("x", true, false, false, true, true, false),
   	      new AttributInfo("y", true, false, false, true, true, false)
   	    }),
   	  }, 
       null), // Keine Mengen definiert
    });

  /**
   * Name des HOT - wird in der Navigationsdarstellung verwendet
   */
  private String _name;
  
  /**
   * Die Systemobjekt-Typen, die von diesem HOT verwendet werden
   */
  private List<String> _typen = new Vector<String>();
  
  /**
   * Die Typinfos zu den Systemobjekttypen
   */
  private Map<String, TypInfo> _typInfos = new HashMap<String, TypInfo>();
  
    
  /**
   * Erzeugen eines neuen HOT_TLS
   * 
   * @param name Name des HOT
   * @param typInfos Systemobjekt-Typen und zugehrige Informationen, 
   * die von diesem HOT verwendet werden
   */
  private HOT_TLS(String name, TypInfo[] typInfos)
  {
    _name = name;
    if (typInfos != null)
    {  
    	for (TypInfo  info: typInfos)
    	{	
    		_typen.add(info.getTyp());
    		_typInfos.put(info.getTyp(), info);
    	}	
    }  
  }
  
//  /**
//   * Erzeugen eines neuen HOT_TLS auf Basis eines bestehenden HOT_TLS
//   * Es werden alle Infos aller Systemobjekttypen des bergeordneten HOT an
//   * den neuen HOT bertragen
//   * 
//   * @param name Name des HOT
//   * @param uebergeordneterHOT der HOT, der beerbt werden soll
//   */
//  private HOT_TLS(String name, HOT_TLS uebergeordneterHOT, String oberTyp, String unterTyp, boolean alleTypen)
//  {
//  	if (alleTypen)
//  	{	
//	  	for (String typ: uebergeordneterHOT.getTypen())
//	  	{	
//	  		TypInfo typInfo = uebergeordneterHOT.getTypInfo(typ);
//		    
//		    List<String> atgs = typInfo.getAtgs();
//		    AtgInfo atgInfo[] = new AtgInfo[atgs.size()];
//		    for (int i = 0; i < atgInfo.length; i += 1)
//		    {
//		    	atgInfo[i] = typInfo.getAtgInfo(atgs.get(i)); 
//		    }
//		    
//		    List<String> mengen = typInfo.getMengen();
//		    MengeInfo mengeInfo[] = new MengeInfo[mengen.size()];
//		    for (int i = 0; i < mengeInfo.length; i += 1)
//		    {
//		    	mengeInfo[i] = typInfo.getMengeInfo(mengen.get(i)); 
//		    }
//		    
//		    if (typ.equals(oberTyp))
//		    {	
//			    _typen.add(unterTyp);
//			    _typInfos.put(unterTyp, new TypInfo(unterTyp, atgInfo, mengeInfo));
//		    }
//		    else
//		    {	
//			    _typen.add(typ);
//			    _typInfos.put(typ, new TypInfo(typ, atgInfo, mengeInfo));
//		    }
//	  	}
//  	}
//  	else
//  	{
//			TypInfo typInfo = uebergeordneterHOT.getTypInfo(oberTyp);
//			List<String> atgs = typInfo.getAtgs();
//			AtgInfo atgInfo[] = new AtgInfo[atgs.size()];
//			for (int i = 0; i < atgInfo.length; i += 1)
//			{
//				atgInfo[i] = typInfo.getAtgInfo(atgs.get(i)); 
//			}
//			List<String> mengen = typInfo.getMengen();
//			MengeInfo mengeInfo[] = new MengeInfo[mengen.size()];
//			for (int i = 0; i < mengeInfo.length; i += 1)
//			{
//				mengeInfo[i] = typInfo.getMengeInfo(mengen.get(i)); 
//			}
//			 
//			_typen.add(unterTyp);
//			_typInfos.put(unterTyp, new TypInfo(unterTyp, atgInfo, mengeInfo));
//		}
//  	
//    _name = name;
//  }
  
  /**
   * Text-Reprsentation fr diesen HOT
   * 
   * @return den Namen des HOT - wird in der Navigationsdarstellung verwendet
   */
  @Override
  public String toString()
  {
    return getName();
  }
    
  /**
	 * @see IHierarchieObjektTyp#getName()
	 */
  public String getName()
  {
    return _name;
  }
  
  /**
	 * @see IHierarchieObjektTyp#getSymbolName()
	 */
  public String getSymbolName()
  {
   	return super.toString();
  }
  
  /**
	 * @see IHierarchieObjektTyp#getTypen()
	 */
  public List<String> getTypen()
  {
    return Collections.unmodifiableList(_typen); 
  }
  
  /**
	 * @see IHierarchieObjektTyp#getTypInfo(String)
	 */
  public TypInfo getTypInfo(String typ)
  {
  	return _typInfos.get(typ);
  }
  
  /**
	 * @see IHierarchieObjektTyp#getTypInfo(SystemObjectType)
	 */
  public TypInfo getTypInfo(SystemObjectType sot)
  {
  	TypInfo result = getTypInfo(sot.getPid());
  	
  	if (result == null && !sot.isBaseType())
  	{
  		for (SystemObjectType superTyp: sot.getSuperTypes())
  		{
  			if (superTyp.isConfigurating())
  			{
  				 result = getTypInfo(superTyp);
  				 if (result != null)
  				 {
  					 break;
  				 }
  			}
  		}
  	}
  	
  	return result;
  }

  @Override
  public boolean isSystemObjektVorhanden()
  {
    // TODO Auto-generated method stub
    return false;
  }
  
  public boolean isSystemObjektNichtAenderbar()
  {
  	return isSystemObjektVorhanden();
  }
}	
