/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkatlsae;

import java.util.Arrays;
import java.util.Collection;
import java.util.List;
import java.util.Vector;

import org.eclipse.core.runtime.Assert;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.MultiStatus;
import org.eclipse.core.runtime.Status;

import de.bsvrz.buv.plugin.tkabasis.Activator;
import de.bsvrz.buv.plugin.tkabasis.DataFactory;
import de.bsvrz.buv.plugin.tkabasis.LookupFactory;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.AtgInfo;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.HODaten;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.HOHierarchie;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.HOMitStatus;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.HierarchieObjekt;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchie;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchieManager;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchieObjektTyp;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.MengeInfo;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.SystemObjektDaten;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.TypInfo;
import de.bsvrz.buv.plugin.tkabasis.pruefungen.IPruefungen;
import de.bsvrz.buv.plugin.tkabasis.pruefungen.LoeschenChecker;
import de.bsvrz.buv.plugin.tkabasis.pruefungen.PruefungenTools;
import de.bsvrz.buv.plugin.tkabasis.regeln.IBildungsRegeln;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.AttributeGroupUsage;
import de.bsvrz.dav.daf.main.config.ConfigurationArea;
import de.bsvrz.dav.daf.main.config.ConfigurationChangeException;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.ConfigurationObjectType;
import de.bsvrz.dav.daf.main.config.NonMutableSet;
import de.bsvrz.dav.daf.main.config.ObjectLookup;
import de.bsvrz.dav.daf.main.config.ObjectSet;
import de.bsvrz.dav.daf.main.config.ObjectSetType;
import de.bsvrz.dav.daf.main.config.ObjectTimeSpecification;
import de.bsvrz.dav.daf.main.config.ReferenceType;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.puk.config.configFile.datamodel.ConfigDataModel;
import de.bsvrz.puk.config.configFile.datamodel.ConfigSystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

public class HierachieManagerTLSAE implements IHierarchieManager
{
	 /**
   * Debug-Logger fr Logging-Ausgaben
   */
	private final static Debug _debug = Debug.getLogger();
	
  private static IHierarchieObjektTyp[] EBENE1 = new IHierarchieObjektTyp[] {HOT_TLSAE.VIZ, HOT_TLSAE.VRZ, HOT_TLSAE.UZ};                                                                      
                
  private IHierarchie _hierarchie;
	
  public IHierarchie getHierarchie()
	{
		return _hierarchie;
	}
  
	public void setHierarchie(IHierarchie hierarchie)
	{
		_hierarchie = hierarchie;
	}
  
  public void bestimmeUntergeordneteHOs(Object objekt)
  {
  	if (objekt instanceof ConfigurationArea)	
    {
			ConfigurationArea kb = (ConfigurationArea)objekt;
					
			for (IHierarchieObjektTyp hot: EBENE1)
			{
				String typPid = hot.getTypen().get(0);
			  SystemObjectType  typ = _hierarchie.getDataModel().getType(typPid);
			  
			  if (typ == null)
			  {
			  	throw new RuntimeException("Typ " + typPid + " wurde im aktuellen Datemmodell nicht gefunden");
			  }
				
		    Collection<SystemObjectType> typeSpez = new Vector<SystemObjectType>(1);
		    typeSpez.add(typ);
		    
		    for (SystemObject so: kb.getObjects(typeSpez, ObjectTimeSpecification.valid()))
		    {
		    	ConfigurationObject ko = (ConfigurationObject)so;
		    	if (!(ko.getNotValidSince() >= ko.getConfigurationArea().getModifiableVersion()))
		    	// Objekt wird in zuknftiger Version nicht ungltig	
		    	{		
		        HierarchieObjekt ho = new HierarchieObjekt(hot, new SystemObject[]{ so }, EBENE1);
		        _hierarchie.objektHinzufuegen(kb, ho);
		        bestimmeUntergeordneteHOs(ho);
		    	}  
		    }
		    
		    for (SystemObject so: kb.getNewObjects())
		    {
		    	if (so.isOfType(typ))
    			{
		    		HierarchieObjekt ho = new HierarchieObjekt(hot, new SystemObject[]{ so }, EBENE1);
			      _hierarchie.objektHinzufuegen(kb, ho);
			      bestimmeUntergeordneteHOs(ho);	
    			}
		    }
		  }
    }
  }
  
  public IHierarchieObjektTyp[] getUntergeordneteHOTs(Object objekt)
  {
  	if (objekt instanceof ConfigurationArea)
  	{
  		return EBENE1;
  	}
  	else
  	{
  		return null;
  	}
  }
  
  private List<ObjectSet> erzeugeMengen(IHierarchieObjektTyp hot, SystemObjectType typ, 
  		ConfigurationArea kb, MultiStatus status)
  {
  	List<ObjectSet> result = new Vector<ObjectSet>();
		
		for (String mengenName: hot.getTypInfo(typ.getPid()).getMengen())
		{
			MengeInfo info = hot.getTypInfo(typ.getPid()).getMengeInfo(mengenName);
			
			SystemObjectType mengenTyp = _hierarchie.getDataModel().getType(info.getTyp());
			ObjectSet menge;
			try
			{
				menge = (ObjectSet)kb.createConfigurationObject((ConfigurationObjectType)mengenTyp, "", mengenName, null);
				
				status.add(new Status(Status.OK, Activator.PLUGIN_ID, 
						"Menge " + mengenName + " vom Typ " + mengenTyp.getPid() + " erzeugt"));
				
			}
			catch (ConfigurationChangeException e)
			{
				status.add(new Status(Status.ERROR, Activator.PLUGIN_ID, "Fehler beim Erzeugen der Menge " 
						+ mengenName + " vom Typ " + mengenTyp.getName() + " erzeugt", e));
				return result;
			}
			
			result.add(menge);
		}
		
		return result;
  }
  
  private SystemObject erzeugeSystemObjekt(String pid, ConfigurationObjectType typ, ConfigurationArea kb,
  		List<ObjectSet> mengen, MultiStatus status)
  {
	  SystemObject result = null;
		
		try
		{
			result = kb.createConfigurationObject(typ, pid, "", mengen);
						
			if (mengen != null && !mengen.isEmpty())
			{
				MultiStatus unterStatus = new MultiStatus(Activator.PLUGIN_ID, IStatus.OK,
						"Systemobjekt " +	pid + " vom Typ " + typ.getPid() + " erzeugt", null);
				status.add(unterStatus);
						
				for (ObjectSet set: mengen)
				{
					unterStatus.add(new Status(Status.OK, Activator.PLUGIN_ID,	"Menge " + set.getName() + 
							" dem Systemobjekt " + pid + " vom Typ " + typ.getPid() + " zugewiesen"));
				}
			}
			else
			{
				status.add(new Status(Status.OK, Activator.PLUGIN_ID, 
						"Systemobjekt " + pid + " vom Typ " + typ.getPid() + " erzeugt"));
			}
		}
		catch (ConfigurationChangeException e)
		{
			status.add(new Status(Status.ERROR, Activator.PLUGIN_ID, 
					"Systemobjekt " + pid + " vom Typ " + typ.getPid() + " konnte nicht erzeugt werden", e));
		}
		
		return result;
  }
   
   
  public HOMitStatus erzeugeNeuesHO(ConfigurationArea kb, Object vater, String pid[], IHierarchieObjektTyp hot, boolean pruefePid)
  {
  	ConfigDataModel dataModel = _hierarchie.getDataModel();
		
		ConfigurationObjectType typ = (ConfigurationObjectType)dataModel.getType(hot.getTypen().get(0));
		if (typ == null)
		{
			throw new IllegalArgumentException("Fr " + hot.getTypen().get(0) + " kann kein Typ im Datenmodell bestimmt werden");
		}
		
		HierarchieObjekt ho = null;
		MultiStatus status = new MultiStatus(Activator.PLUGIN_ID, Status.OK,
				"Erzeuge neues Hierarchieobjekt vom Typ " + hot, null);
	
		if (pruefePid)
		{	
			status.add(new PruefungenTools(dataModel).pruefePid(pid[0]));
		}	
		
		if (!status.matches(IStatus.ERROR | IStatus.CANCEL))
		{
			List<ObjectSet> mengen = erzeugeMengen(hot, typ, kb, status);
			
			if (!status.matches(IStatus.ERROR | IStatus.CANCEL))
			{
				SystemObject so = erzeugeSystemObjekt(pid[0], typ, kb, mengen, status);
				
				if (!status.matches(IStatus.ERROR | IStatus.CANCEL))
				{
					ho = new HierarchieObjekt(hot, new SystemObject[] { so }, EBENE1);
					getHierarchie().objektHinzufuegen(vater, ho);
				}
			}
		}	
		
		return new HOMitStatus(ho, status);
  }
  
  public HOMitStatus kopiereHO(HierarchieObjekt ho, ConfigurationArea kb, Object neuerVater, 
			String neuePid[],	boolean rekursiv, boolean isWurzel)
  {
  	return kopiereHO(ho, kb, neuerVater, neuePid, rekursiv, true, true);
  }
	  
  private HOMitStatus kopiereHO(HierarchieObjekt ho, ConfigurationArea kb, Object neuerVater, 
  		String neuePid[],	boolean rekursiv, boolean anwendenBR, boolean pruefePid)
  {
  	MultiStatus status = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Kopiere Hierarchieobjekt " + ho, null);
  
  	String[] pid = neuePid;
  	if (pid == null)
  	{	
  		pid = new String[1];
  		pid[0] = ho.getSystemObjekt(0).getPid();
  	}	
  	
  	HOMitStatus result = erzeugeNeuesHO(kb, neuerVater, pid, ho.getHierarchieObjektTyp(), pruefePid);
  	HierarchieObjekt neu = result.getHO();
  	status.add(result.getStatus());
  	  	
  	if (!status.matches(IStatus.ERROR | IStatus.CANCEL))
		{
  		status.merge(kopiereDaten(ho, result.getHO(), anwendenBR, anwendenBR));
		}
  	
  	if (!status.matches(IStatus.ERROR | IStatus.CANCEL) && rekursiv)
		{
  		for (HierarchieObjekt lauf: _hierarchie.getUntergeordneteObjekte(ho))
  		{
  			MultiStatus meldung = new MultiStatus(Activator.PLUGIN_ID, Status.OK, 
  					"Kopiere untergeordnetes Hierarchieobjekt " + lauf + " zu " + ho, null);
  			result = kopiereHO(lauf, kb, neu, null, true, anwendenBR, pruefePid);
  			meldung.merge(result.getStatus());
  			status.add(meldung);
  			if (status.matches(IStatus.ERROR | IStatus.CANCEL))
				{
  				break;
				}
  		}
		}
  	
  	return new HOMitStatus(neu, status);
  }
  
  
  private MultiStatus kopiereDaten(HierarchieObjekt srcHO, HierarchieObjekt destHO, 
  		boolean bildungsRegelnFuerName, boolean bildungsRegelnFuerAtg)
  {
  	if (!srcHO.getHierarchieObjektTyp().equals(destHO.getHierarchieObjektTyp()))
  	{
  		throw new IllegalArgumentException("Beide bergebenen Hierarchieobjekte mssen vom gleichen Hierarchieobjekttyp sein");
  	}
  	
		int index = 0;
		MultiStatus status = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Kopiere Daten von " + srcHO +	" nach " + destHO, null);
		IBildungsRegeln br = DatenSpeicher.getInstanz().getBildungsRegeln();
		ConfigDataModel dataModel = _hierarchie.getDataModel();
		
		ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(dataModel);
		
		for (SystemObject src: srcHO.getSystemObjekte())
		{
			SystemObject dest = destHO.getSystemObjekte().get(index);
			String name;
			if (bildungsRegelnFuerName)
			{
				name = br.getName(dest);
			}
			else
			{
				name = src.getName();
			}
			
			try
			{
				dest.setName(name);
				status.add(new Status(Status.OK, Activator.PLUGIN_ID,	"Setze Name \"" + name + 
						"\" fr Systemobjekt " +	dest.getPid()));
			}
			catch (ConfigurationChangeException e)
			{
				status.add(new Status(Status.ERROR, Activator.PLUGIN_ID,	"Name \"" + name + 
						"\" fr Systemobjekt " +	dest.getPid() + " konnte nicht gesetzt werden", e));
				return status;
			}
			
			for (AttributeGroupUsage atgu: src.getUsedAttributeGroupUsages())
			{
				Data dataSrc = ((ConfigSystemObject)src).getConfigurationData(atgu, lookup);
				if (dataSrc != null)
				{
					try
					{
						Data dataDest = DataFactory.createModifiableCopy(dataSrc, lookup);
						if (bildungsRegelnFuerAtg)
						{
							dataDest = br.getDatensatz(destHO, dest, atgu.getAttributeGroup(), dataDest, true);
						}
						dest.setConfigurationData(atgu, dataDest);
	  				status.add(new Status(Status.OK, Activator.PLUGIN_ID,	"Kopiere Daten der ATG-Verwendung " +
	  						atgu.getAttributeGroup().getPid() + "/" + atgu.getAspect().getPid() + 
	  						" fr Systemobjekt " +	dest.getPid())); 
					}
					catch (ConfigurationChangeException e)
					{
						status.add(new Status(Status.ERROR, Activator.PLUGIN_ID,	"Daten der ATG-Verwendung " +
								atgu.getAttributeGroup().getPid() + "/" + atgu.getAspect().getPid() + 
								" fr Systemobjekt " +	dest.getPid() + " konnten nicht kopiert werden", e));
						return status;
					}
				}
			}
			
			index++;
		}
		
		return status;
  }
    
  
  public HOMitStatus verschiebeHO(HierarchieObjekt ho, Object neuerVater)
  {
  	Object alterVater = _hierarchie.getUebergeordnetesObjekt(ho);
  	ConfigurationArea srcKB = _hierarchie.getKonfigurationsBereich(alterVater);
  	Assert.isNotNull(srcKB, "Der Konfigurationsbereich zu " + alterVater + " darf nicht null sein");
  	ConfigurationArea destKB = _hierarchie.getKonfigurationsBereich(neuerVater);
  	Assert.isNotNull(destKB, "Der Konfigurationsbereich zu " + neuerVater + " darf nicht null sein");
  	
  	MultiStatus status = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Verschiebe Hierarchieobjekt " + ho, null);
  	
  	HierarchieObjekt neu = null;
  	
  	if (srcKB.equals(destKB)) // HO wird im gleichen kb verschoben
  	{
  		if (neuerVater.equals(alterVater))
  		{
  			status.add(new Status(Status.INFO, Activator.PLUGIN_ID,	"Objekt " + ho +
				" kann nicht in sich selbst verschoben werden"));
  		}
  	}
  	else // HO wird in einen anderen kb verschoben
  	{
    	HOHierarchie hoHierarchie = new HOHierarchie(ho, _hierarchie);
    	  
  	  status.add(loescheHO(ho));
  	  
  	  if (!status.matches( IStatus.ERROR | IStatus.CANCEL ))
  	  {
  	    HOMitStatus result = kopiereHORekursiv(hoHierarchie.getWurzel(), destKB, neuerVater,
  	        hoHierarchie);
  	    status.add( result.getStatus() );
  	    neu = result.getHO();
  	  }
 		}
  	return new HOMitStatus( neu, status );
  }
  
  private HOMitStatus kopiereHORekursiv(HierarchieObjekt ho, ConfigurationArea kb, Object vater,
      HOHierarchie hierarchie)
  {
    MultiStatus meldung;
    
    if (ho == hierarchie.getWurzel())
    {  
      meldung = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Kopiere Hierarchieobjekt " + ho, null);
    }
    else
    {
      meldung = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Kopiere untergeordnetes Hierarchieobjekt " + 
          ho + " zu " + vater, null);
    }
    
    HOMitStatus result = kopiereHO(ho, kb, vater, null, false, false, false);
    meldung.merge( result.getStatus() );
    HierarchieObjekt neu = result.getHO();

    if (neu != null)
    {  
      for (HierarchieObjekt kind: hierarchie.getUntergeordneteObjekte( ho ))
      {
        result = kopiereHORekursiv(kind, kb, neu, hierarchie);
        meldung.add( result.getStatus() );
      }
    }
    
    return new HOMitStatus(neu, meldung);
  }
    
	public IStatus loescheHO(HierarchieObjekt ho)
	{
		MultiStatus status = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Lsche Hierarchieobjekt " + ho, null);
				
		loescheHO(ho, status);
		
		return status;
		//Activator.getDefault().getLog().log(status);
	}
  
  private void loescheHO(HierarchieObjekt ho, MultiStatus status)
  {
  	IHierarchieObjektTyp hot = ho.getHierarchieObjektTyp();
  	
  	Object vater = getHierarchie().getUebergeordnetesObjekt(ho);
  	if (vater == null)
  	{
  		throw new IllegalArgumentException("Kein bergeordnetes Objekt fr " + ho +
  				" gefunden in der Hierarchie TLS hnliche Kommunikationspartner");
  	}
  	
  	if (Arrays.asList(EBENE1).contains(hot) && !(vater instanceof ConfigurationArea))
  	{
  		throw new IllegalArgumentException("Als bergeordnetes Objekt ist nur ein Konfigurationsbereich zulssig");
  	}
  	
  	while (!getHierarchie().getUntergeordneteObjekte(ho).isEmpty())
  	{
  		HierarchieObjekt objekt = getHierarchie().getUntergeordneteObjekte(ho).get(0);
  		MultiStatus unterStatus = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Lsche Hierarchieobjekt " + objekt, null);
  		
  		loescheHO(objekt, unterStatus);
  		
  		status.add(unterStatus);
  		
  		if (unterStatus.matches(IStatus.CANCEL | IStatus.ERROR))
  		{
  			//break;
  			return;
  		}
  	}
  	
  	ConfigurationObject ko = (ConfigurationObject)ho.getSystemObjekte().get(0);
  	
  	// Lsche Objekt
  	try
  	{
  		ko.invalidate();
  		  		
  		status.add(new Status(Status.OK, Activator.PLUGIN_ID, "Konfigurationsobjekt " + 
  				ko.getPid() + " vom Typ " + ko.getType().getName() + " gelscht."));
  	}
  	catch (ConfigurationChangeException e)
  	{
  		status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Konfigurationsobjekt " + 
  				ko.getPid() + " vom Typ " + ko.getType().getName() + " konnte nicht gelscht werden.", e));
  		return;
  	}
  	catch (RuntimeException e)
  	{
  		status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Konfigurationsobjekt " + 
  				ko.getPid() + " vom Typ " + ko.getType().getName() + " konnte nicht gelscht werden.", e));
  		return;
  	}
  	
  	getHierarchie().objektEntfernen(vater, ho);
  }
  
  public IStatus pruefeDaten(HierarchieObjekt ho, HODaten daten)
	{
		IHierarchieObjektTyp hot = ho.getHierarchieObjektTyp();
		IPruefungen pruefInstanz = DatenSpeicher.getInstanz().getPruefungen();
		MultiStatus result = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Prfe Hierarchieobjekt " + ho, null);
		
		// Prfe Hierarchieobjekt
		result.merge(pruefInstanz.pruefeHierarchieObjekt(ho, daten));
		
		// Prfe auf offene Referenzen bei PID-nderungen
		LoeschenChecker loeschenChecker = new LoeschenChecker(this);
		List<SystemObject> liste = new Vector<SystemObject>();
    StringBuffer aend = new StringBuffer();
		StringBuffer loesch = new StringBuffer();
		
		for (SystemObject so: ho.getSystemObjekte())
		{
		 SystemObjektDaten soDaten = daten.getSystemObjektDaten(so);
		 
		 if (!so.getPid().equals(soDaten.getPid()))
		 {
			 liste.add(so);
			 if (aend.length() > 0)
			 {
				 aend.append(", ");
			 }
			 aend.append(soDaten.getPid());
			 if (loesch.length() > 0)
			 {
				 loesch.append(", ");
			 }
			 loesch.append(soDaten.getPid());
		 }
		}
		
		if (liste.size() > 0)
		{
			 MultiStatus status = new MultiStatus(Activator.PLUGIN_ID, Status.OK,  "Durch PID-nderung(en) auf " + 
					 aend + " werden folgende Objekte gelscht: " + loesch, null);
			 
//			 status.add(loeschenChecker.bestimmeOffeneReferenzen(liste));
			 
			 Collection<SystemObjectType> typen = loeschenOffeneReferenzenVerboten(ho.getHierarchieObjektTyp());
       boolean flag = verwendeZuLoeschendeObjekteZurPruefungOffeneReferenzen();
       status.add(loeschenChecker.bestimmeOffeneReferenzen(liste, typen, flag));
       
			 result.add(status);
		 }
		
		// Prfe ATG-Daten
		for (SystemObject so: ho.getSystemObjekte())
  	{
			 SystemObjektDaten soDaten = daten.getSystemObjektDaten(so);
			 if (soDaten != null)
			 {
				 MultiStatus objektStatus;
				 if (ho.getSystemObjekte().size() > 1)
				 {
					 objektStatus = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Prfe ATG-Daten fr Systemobjekt " + 
							 so.getPid() + " vom Typ " + so.getType().getPid(), null);
				 }
				 else
				 {
					 objektStatus = result;
				 }
				 
				 // Prfe ATG-Daten  fr das Systemobjekt
				 TypInfo typInfo = hot.getTypInfo(so.getType().getPid());
				 if (typInfo != null)
				 {
					 for (String atgPid: typInfo.getAtgs())
					 {
						 AtgInfo atgInfo = typInfo.getAtgInfo(atgPid);
						 if (atgInfo != null)
						 {
							 Data data = soDaten.getAtgDaten(atgPid);
							 if (data == null && (atgInfo.isMandatory() || !atgInfo.isOptional()))
							 {
								 objektStatus.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Fr die nicht-optionale ATG " +
										 atgPid + " sind keine Daten vorhanden"));					 
							 }
							 if (data != null)
							 {
								 MultiStatus atgStatus = new MultiStatus(Activator.PLUGIN_ID, Status.OK, 
										 "Prfe Daten der ATG " + atgPid, null);
								 atgStatus.merge(pruefInstanz.pruefeDaten(ho, so, atgInfo, data));
								 								 								 
								 objektStatus.add(atgStatus);
							 }
						 }
						 else
						 {
								// Kein AtgInfo fr den aktuellen Typ-Info-Typ im HOT fr aktuellen Systemobjekttyp
							 throw new IllegalStateException("AtgInfo fr ATG " + atgPid + " fehlt in TypInfo fr " + 
									 so.getType().getPid() + " im HierarchieObjektTyp " + hot);
						 }
					 }
				 }
				 else
				 {
				// Keine Typ-Info im HOT fr aktuellen Systemobjekttyp
					 throw new IllegalStateException("TypInfo fehlt fr  " + so.getType().getPid() + 
							 " im HierarchieObjektTyp " + hot);
				 }
				 if (result != objektStatus)
				 {
					 result.add(objektStatus);
				 }
				 
			 }
			 else
			 {
				 // Keine Daten fr Systemobjekt so bergeben
				 throw new IllegalStateException("Daten fr Systemobjekt " + so.getPid() +
						 " erforderlich, aber nicht bergeben");
			 }
  	}
  	
  	return result;
	}    
  
  protected ConfigurationObject dupliziereObjekt(ConfigurationObject ko, String neuePid)
	throws ConfigurationChangeException
{
	List<ObjectSet> sets = new Vector<ObjectSet>();
	ConfigurationObject neu = null;
	ConfigurationObject result = null;
	
	try
	{
	  for (ObjectSet set: ko.getObjectSets())
	  {
	    ObjectSetType setTyp = set.getObjectSetType();
	    assert(setTyp.getReferenceType() != ReferenceType.COMPOSITION);
	    ObjectSet menge = (ObjectSet)ko.getConfigurationArea().createConfigurationObject(set.getObjectSetType(), 
	        "", set.getName(), null);
	    List<SystemObject> elemente = set instanceof NonMutableSet ? 
	        ((NonMutableSet)set).getElementsInModifiableVersion() : set.getElements();
	    menge.add(elemente.toArray(new SystemObject[elemente.size()]));
	    sets.add(menge);
	  }
	  
	  neu = ko.getConfigurationArea().createConfigurationObject((ConfigurationObjectType)ko.getType(),
	      neuePid == null ? ko.getPid() : neuePid, ko.getName(), sets);
	  
	  ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion((ConfigDataModel)ko.getDataModel());
	  
	  for (AttributeGroupUsage atgu: ko.getUsedAttributeGroupUsages())
	  {
	    Data dataSrc = ((ConfigSystemObject)ko).getConfigurationData(atgu, lookup);
	    if (dataSrc != null)
	    {
	      Data dataDest = DataFactory.createModifiableCopy(dataSrc, lookup);
	      neu.setConfigurationData(atgu, dataDest);
	    }
	  }
	  result = neu;
	}
	finally
	{
	  // Rollback
	  if (result == null)
	  { 
	    for (ObjectSet set: sets)
	    {
	      set.invalidate();
	    }
	    if (neu != null)
	    {
	      neu.invalidate();
	    }
	  } 
	}

	return result;
}

private ConfigurationObject behandleSystemObjektAendeung(HierarchieObjekt ho, ConfigurationObject ko, 
    String neuePid) throws ConfigurationChangeException
{
  ConfigurationObject result = dupliziereObjekt(ko, neuePid);
  
  ko.invalidate();
  
  return result;
} 
  
  public IStatus uebernehmeDaten(HierarchieObjekt ho, HODaten daten)
  {
  	MultiStatus status = new MultiStatus(Activator.PLUGIN_ID, Status.OK, 
  			"bernehme bearbeitete Daten fr Hierarchieobjekt " + ho, null);

  	List<SystemObject> objektListe = daten.getSystemObjekte();

//  	DatenModellUpdateListener.getInstanz().notifyDatenModellUpdateListener(objektListe);
  	
  	for (int i = 0; i < objektListe.size(); i += 1)
  	{
  		SystemObject so = objektListe.get(i);
  		
  		if (ho.getSystemObjekte().contains(so))
			{
  			SystemObjektDaten soDaten = daten.getSystemObjektDaten(so);
  			
  			// PID
  			if (!so.getPid().equals(soDaten.getPid()))
  			{
          try
          {
	  				so = behandleSystemObjektAendeung(ho, (ConfigurationObject)so, soDaten.getPid());
	  				ho.setSystemObjekt(i, so);
	  				status.add(new Status(IStatus.OK, Activator.PLUGIN_ID, 
	  						"Neues Objekt mit PID " + so.getPid() + 
	  						" vom Typ " + so.getType().getPid() + " erzeugt"));
          }
          catch (ConfigurationChangeException e)
        	{
          	status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, 
      					"Fehler beim Behandeln der PID-nderung auf " + soDaten.getPid() +
      					" fr Objekt " + so.getPid(), e));
        	}
  			}

  			// Name
  			if (so.getName() == null || !so.getName().equals(soDaten.getName()))
  			{
	  			try
	      	{
	  				so.setName(soDaten.getName());
	  				status.add(new Status(IStatus.OK, Activator.PLUGIN_ID, 
	   						"Name \"" + soDaten.getName() + "\" bernommen"));
	      	}
	  	   	catch (ConfigurationChangeException e)
	      	{
	  	   		status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, 
	   						"Name \"" + soDaten.getName() + "\" konnte nicht bernommen werden" , e));
	      	}
  			} 	
  		
  	   	// ATGs
  			List<String> atgList = ho.getHierarchieObjektTyp().getTypInfo(so.getType().getPid()).getAtgs();
  			
		    for (String atgPid: atgList)
    		{
		    	AttributeGroup atg = _hierarchie.getDataModel().getAttributeGroup(atgPid);
		    	if (atg != null)
		    	{
		    		ConfigDataModel dataModel = _hierarchie.getDataModel();
		    		ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(dataModel);
		    		
		    		//Data ist = so.getConfigurationData(atg);
		        Data ist = ((ConfigSystemObject)so).getConfigurationData(atg, lookup);
		        		        
		    		Data soll = soDaten.getAtgDaten(atgPid);
		  
	          if (soll != null)
	          {
	          	if (ist == null || soDaten.isDatenGeaendert(atgPid))
	          	{	
		          	if (!HierarchieObjekt.isAttributgruppeAenderbar(so, atg))
				    		{
		          		 try
		               {
		     	  				so = behandleSystemObjektAendeung(ho, (ConfigurationObject)so, so.getPid());
		     	  				ho.setSystemObjekt(i, so);
		     	  				status.add(new Status(IStatus.OK, Activator.PLUGIN_ID,
		     	  						"ATG-Verwendung " + AttributeGroupUsage.Usage.OptionalConfigurationData +
		     	  						" bedingt Erzeugung neues Objekt mit PID " + so.getPid() + " vom Typ " + 
		     	  						so.getType().getPid()));
		               }
		               catch (ConfigurationChangeException e)
		             	{
		               	status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, 
		               			"Bei der durch ATG-Verwendung " + AttributeGroupUsage.Usage.OptionalConfigurationData +
		     	  						" bedingten Erzeugung neues Objekts mit PID " + so.getPid() + " vom Typ " +	
		     	  						so.getType().getPid() + " ist ein Fehler aufgetreten", e));
		               	continue;
		             	}
		          		
//		          		status.add(new Status(IStatus.INFO, Activator.PLUGIN_ID, 
//				   						"Eventuelle nderungen an konfigurierenden Daten der ATG " + atgPid +
//				   						" werden IGNORIERT, da wegen ATG-Verwendung (" + 
//				   						AttributeGroupUsage.Usage.OptionalConfigurationData + ") nicht mglich"));
//				    			continue;
		               
				    		}
		          	try
		          	{
		          		so.setConfigurationData(atg, soll);
		          		status.add(new Status(IStatus.OK, Activator.PLUGIN_ID, 
		       						"Konfigurierende Daten der ATG " + atgPid + " bernommen"));
		          	}
		          	catch (ConfigurationChangeException e)
		          	{
		          		status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, 
		       						"Fehler beim bernehmen konfigurierender Daten der ATG " + atgPid, e));
		          	}
	          	}	
	          }
	          else // Keine Daten fr diese ATG bergeben => vorhanden Daten lschen
	          {
	          	if (ist != null)
	          	{
	          		if (!HierarchieObjekt.isAttributgruppeAenderbar(so, atg))
				    		{
	          			 try
		               {
		     	  				so = behandleSystemObjektAendeung(ho, (ConfigurationObject)so, so.getPid());
		     	  				ho.setSystemObjekt(i, so);
		     	  				status.add(new Status(IStatus.OK, Activator.PLUGIN_ID,
		     	  						"ATG-Verwendung " + AttributeGroupUsage.Usage.OptionalConfigurationData +
		     	  						" bedingt Erzeugung neues Objekt mit PID " + so.getPid() + " vom Typ " + 
		     	  						so.getType().getPid()));
		               }
		               catch (ConfigurationChangeException e)
		             	{
		               	status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, 
		               			"Bei der durch ATG-Verwendung " + AttributeGroupUsage.Usage.OptionalConfigurationData +
		     	  						" bedingten Erzeugung neues Objekts mit PID " + so.getPid() + " vom Typ " +	
		     	  						so.getType().getPid() + " ist ein Fehler aufgetreten", e));
		               	continue;
		             	}
	          			
//				    			status.add(new Status(IStatus.INFO, Activator.PLUGIN_ID, 
//				   						"Eventuelle nderungen an konfigurierenden Daten der ATG " + atgPid +
//				   						" werden IGNORIERT, da wegen ATG-Verwendung (" + 
//				   						AttributeGroupUsage.Usage.OptionalConfigurationData + ") nicht mglich"));
//				    			continue;
		               
				    		}
	          		try
		          	{
		          		so.setConfigurationData(atg, null);
		          		status.add(new Status(IStatus.OK, Activator.PLUGIN_ID, 
		       						"Konfigurierende Daten der ATG " + atgPid + " gelscht"));
		          	}
		          	catch (ConfigurationChangeException e)
		          	{
		          		status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, 
		       						"Fehler beim Lschen konfigurierender Daten der ATG " + atgPid, e));
		          	}
	          	}
	          }
		    	}
		    	else
		    	{
		    		_debug.warning("Bearbeitung Objekt " + so.getPid() + 
		    				": Keine Attributgruppe im Datenmodell gefunden fr " + atgPid);
		    	}
    		}	
			}   	
  	}    
  	
  	return status;
  }

  @Override
  public boolean pruefeNeuanlageVerboten(Object objekt, IHierarchieObjektTyp hot)
  {
    return false;
  }

  @Override
  public Collection<SystemObjectType> loeschenOffeneReferenzenVerboten(IHierarchieObjektTyp hot)
  {
    return null;
  }

  @Override
  public boolean verwendeZuLoeschendeObjekteZurPruefungOffeneReferenzen()
  {
    return false;
  }
  
  /**
   * Zeigt an, ob das bergebene Hierarchieobjekt beim rekursiven Kopieren als untergeordnetes 
   * Objekt mitkopiert werden soll oder nicht
   *  
   * @param ho das aktuell zu kopierende HO
   * @return <code>true</code>, wenn das aktuell zu kopierende Objekt nicht kopiert werden soll, 
   * <code>false</code> (default) sonst
   */
  public boolean kopierenNichtErlaubt(HierarchieObjekt ho)
  {
  	return false;
  }
 
}
