/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkatlsae;

import java.util.List;

import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.ui.IStartup;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;

import de.bsvrz.buv.plugin.tkabasis.TKABasisComposite;
import de.bsvrz.buv.plugin.tkabasis.TKABasisView;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.AtgInfo;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.TypInfo;
import de.bsvrz.buv.plugin.tkabasis.pruefungen.IPruefungen;
import de.bsvrz.buv.plugin.tkabasis.regeln.IBildungsRegeln;
import de.bsvrz.buv.plugin.tkatlsae.preferences.DefsPara;
import de.bsvrz.buv.rw.rw.menu.ElementRegistrierung;
import de.bsvrz.puk.config.configFile.datamodel.ConfigDataModel;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * {@link IStartup} Implementierung, registriert alle Actions dieses Moduls
 * zwecks Men- und Symbolleistenaufbau bzw. deren Verwaltung.
 * 
 * @version $Revision: 1.2 $ / $Date: 2011/01/17 14:41:24 $ / ($Author: Pittner
 *          $)
 * 
 * @author HofmannJ
 * 
 */
public class Starter implements IStartup {
	/**
	 * Der Debugger
	 */
	private Debug _debug = Debug.getLogger();

	/**
	 * Der Zaehler fr die Views
	 */
	private int _numViewInstances = 0;

	/**
	 * In dieser Methode werden die vom Modul Betriebsmeldungsverwaltung
	 * bereitgestellten Actions generiert und in der Klasse
	 * {@link ElementRegistrierung} ber deren Methode
	 * {@link ElementRegistrierung#registriereDeskriptorAktion(de.bsvrz.buv.rw.rw.menu.deskriptoren.DeskriptorAktion)}
	 * registriert.
	 */
	public void earlyStartup() {
	}

	private void starteView(IWorkbenchPage page) throws PartInitException {
		// String id = DatenSpeicher.getInstanz().holeTKABasisView().ID;
		IPreferenceStore preferences = Activator.getDefault()
				.getPreferenceStore();
		HOT_TLSAE[] hotMqs = HOT_TLSAE.values();

		for (int i = 0; i < hotMqs.length; i++) {
			HOT_TLSAE hotMQ = hotMqs[i];

			boolean istAtgMandatory = true;

			List<String> typen = hotMQ.getTypen();
			for (String typ : typen) {

				TypInfo ti = hotMQ.getTypInfo(typ);
				List<String> atgs = ti.getAtgs();

				for (String atg : atgs) {
					AtgInfo ai = ti.getAtgInfo(atg);
					istAtgMandatory = ai.isMandatory();
					if (!istAtgMandatory) {
						boolean optional = preferences.getBoolean(typ
								+ "," + atg);
						ai.setOptional(optional);		
					}
				}
			}
		}

		String id = TKABasisView.ID;
		TKABasisView view = (TKABasisView) page.showView(id, "MQ",
				IWorkbenchPage.VIEW_CREATE);

		DatenSpeicher.getInstanz().setzeTKABasisView(view);

		// TKABasisView view = DatenSpeicher.getInstanz().holeTKABasisView();

		ConfigDataModel dataModel = DatenSpeicher.getInstanz()
				.holeConfigDataModel();

		String pfad = DatenSpeicher.getInstanz().holeDateiPfad();

		if (DatenSpeicher.getInstanz().getHierarchieManager() == null) {
			DatenSpeicher.getInstanz().setHierarchieManager(
					new HierachieManagerTLSAE());
		}

		if (DatenSpeicher.getInstanz().getBildungsRegeln() == null) {
			String klassenName = Activator.getDefault().getPreferenceStore()
					.getString(DefsPara.KlasseBildungsregeln);
			String defKlassenName = Activator.getDefault().getPreferenceStore()
					.getDefaultString(DefsPara.KlasseBildungsregeln);

			IBildungsRegeln bildungsRegeln = (IBildungsRegeln) TKATLSAEPart
					.ladeKlassenInstanz(klassenName, IBildungsRegeln.class,
							DefsPara.NameBildungsregeln, null, defKlassenName);

			if (bildungsRegeln != null) {
				DatenSpeicher.getInstanz().setBildungsRegeln(bildungsRegeln);
			}
		}

		if (DatenSpeicher.getInstanz().getPruefungen() == null) {
			String klassenName = Activator.getDefault().getPreferenceStore()
					.getString(DefsPara.KlassePruefungen);
			String defKlassenName = Activator.getDefault().getPreferenceStore()
					.getDefaultString(DefsPara.KlassePruefungen);

			IPruefungen pruefungen = (IPruefungen) TKATLSAEPart
					.ladeKlassenInstanz(klassenName, IPruefungen.class,
							DefsPara.NamePruefungen, null, defKlassenName);

			if (pruefungen != null) {
				DatenSpeicher.getInstanz().setPruefungen(pruefungen);
			}
		}

		view.setModelInfo(DatenSpeicher.getInstanz());

		TKABasisComposite tkaComposite = view.getTkaComposite();

		tkaComposite.setConfigDataModel(dataModel, pfad);
	}
}
